# Copyright 2018-2026 Jérôme Dumonteil
# Copyright (c) 2009-2010 Ars Aperta, Itaapy, Pierlis, Talend.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
#
# Authors (odfdo project): jerome.dumonteil@gmail.com
# The odfdo project is a derivative work of the lpod-python project:
# https://github.com/lpod/lpod-python
# Authors: Romain Gauthier <romain@itaapy.com>
#          Hervé Cauwelier <herve@itaapy.com>
#          David Versmisse <david.versmisse@itaapy.com>
#          Jerome Dumonteil <jerome.dumonteil@itaapy.com>

from collections.abc import Iterable
from decimal import Decimal as dec

import pytest

from odfdo.document import Document
from odfdo.table import Cell, Row, Table


@pytest.fixture
def row(samples) -> Iterable[Table]:
    # simpletable :
    #   1	1	1	2	3	3	3
    #   1	1	1	2	3	3	3
    #   1	1	1	2	3	3	3
    #   1   2	3	4	5	6	7
    document = Document(samples("simple_table.ods"))
    table = document.body.get_table(name="Example1")
    yield table.get_row(1)


@pytest.fixture
def row_repeats(samples) -> Iterable[Table]:
    # simpletable :
    #   1	1	1	2	3	3	3
    #   1	1	1	2	3	3	3
    #   1	1	1	2	3	3	3
    #   1   2	3	4	5	6	7
    document = Document(samples("simple_table.ods"))
    table = document.body.get_table(name="Example1")
    yield table.get_row(0)


def test_iter_cells(row):
    assert len(list(row.iter_cells())) == 7


def test_iter_cells_coord_2_None(row):
    assert len(list(row.iter_cells(2, None))) == 5


def test_iter_cells_coord_2_4(row):
    assert len(list(row.iter_cells(2, 4))) == 3


def test_iter_cells_coord_0_3(row):
    assert len(list(row.iter_cells(0, 3))) == 4


def test_iter_cells_coord_0_many(row):
    assert len(list(row.iter_cells(0, 55))) == 7


def test_iter_cells_coord_many_many(row):
    assert len(list(row.iter_cells(100, 55))) == 0


def test_iter_cells_coord_many_none(row):
    assert len(list(row.iter_cells(100, None))) == 0


def test_iter_cells_coord_none_1(row):
    assert len(list(row.iter_cells(None, 1))) == 2


def test_iter_cells_coord_m5_1(row):
    assert len(list(row.iter_cells(-5, 1))) == 2


def test_iter_cells_coord_2_m1(row):
    assert len(list(row.iter_cells(2, -1))) == 0


def test_iter_cells_coord_m5_m1(row):
    assert len(list(row.iter_cells(-5, -1))) == 0


def test_traverse_func(row):
    # compatibility traverse = iter_cells
    assert row.traverse == row.iter_cells


def test_traverse(row):
    # compatibility traverse = iter_cells
    assert len(list(row.traverse())) == 7


def test_traverse_cells_coord(row):
    # compatibility traverse = iter_cells
    assert len(list(row.traverse(2, 4))) == 3


def test_get_cells(row):
    assert len(list(row.get_cells())) == 7


def test_get_cells_property(row):
    assert len(row.cells) == 7


def test_get_cells_on_emty_row():
    row = Row()
    assert len(row.get_cells()) == 0
    assert len(row.cells) == 0
    assert len(row.get_cells((1, 2))) == 0
    assert len(row.get_cells((-2, -3))) == 0
    assert len(row.get_cells((0, 10))) == 0


def test_get_cells_coord(row):
    coord = (0, 8)
    assert len(row.get_cells(coord)) == 7
    coord = "a1:c2"
    assert len(row.get_cells(coord)) == 3
    coord = "a1:a2"
    assert len(row.get_cells(coord)) == 1
    coord = "a1:EE2"
    assert len(row.get_cells(coord)) == 7
    coord = "D1"
    assert len(row.get_cells(coord)) == 0
    coord = "c5:a1"
    assert len(row.get_cells(coord)) == 0
    coord = (5, 6)
    assert len(row.get_cells(coord)) == 2
    coord = (-5, 6)
    assert len(row.get_cells(coord)) == 5
    coord = (0, -1)
    assert len(row.get_cells(coord)) == 7
    coord = (0, -2)
    assert len(row.get_cells(coord)) == 6
    coord = (-1, -1)
    assert len(row.get_cells(coord)) == 1
    coord = (1, 0)
    assert len(row.get_cells(coord)) == 0


def test_get_cells_regex(row):
    coordinates = [cell.x for cell in row.get_cells(content=r"3")]
    expected = [4, 5, 6]
    assert coordinates == expected


def test_get_cells_style(row):
    coordinates = [cell.x for cell in row.get_cells(style=r"ce5")]
    expected = [1, 5]
    assert coordinates == expected


def test_get_cells_cell_type(row):
    cells = row.get_cells(cell_type="all")
    assert len(cells) == 7
    cells = row.get_cells(cell_type="float")
    assert len(cells) == 7
    cells = row.get_cells(cell_type="percentage")
    assert len(cells) == 0
    cells = row.get_cells(cell_type="string")
    assert len(cells) == 0


def test_get_cells_cell_type2(row):
    row.append_cell(Cell(value="bob"), clone=False)
    row.append_cell(Cell(value=14, cell_type="percentage"))
    row.append_cell(Cell(value="bob2"), clone=False)
    cells = row.get_cells(cell_type="all")
    assert len(cells) == 7 + 3
    cells = row.get_cells(cell_type="float")
    assert len(cells) == 7
    cells = row.get_cells(cell_type="percentage")
    assert len(cells) == 1
    cells = row.get_cells(cell_type="string")
    assert len(cells) == 2


def test_get_cells_cell_type_and_coord(row):
    cells = row.get_cells(coord=(0, 5), cell_type="all")
    assert len(cells) == 6
    cells = row.get_cells(coord=(0, 5), cell_type="float")
    assert len(cells) == 6
    cells = row.get_cells(coord=(0, 5), cell_type="percentage")
    assert len(cells) == 0
    cells = row.get_cells(coord=(2, 5), cell_type="string")
    assert len(cells) == 0


def test_get_cells_cell_type_and_coord2(row):
    row.append_cell(Cell(value="bob"), clone=False)
    row.append_cell(Cell(value=14, cell_type="percentage"))
    row.append_cell(Cell(value="bob2"), clone=False)
    cells = row.get_cells(coord=(2, 9), cell_type="all")
    assert len(cells) == 8
    cells = row.get_cells(coord=(3, 9), cell_type="float")
    assert len(cells) == 4
    cells = row.get_cells(coord=(0, 5), cell_type="percentage")
    assert len(cells) == 0
    cells = row.get_cells(coord=(0, 5), cell_type="string")
    assert len(cells) == 0
    cells = row.get_cells(coord=(5, 9), cell_type="percentage")
    assert len(cells) == 1
    cells = row.get_cells(coord=(5, 9), cell_type="string")
    assert len(cells) == 2
    cells = row.get_cells(coord=(8, 9), cell_type="string")
    assert len(cells) == 1


def test_get_cell_alpha(row):
    cell_5 = row.get_cell("F")
    assert cell_5.get_value() == 3
    assert cell_5.text_content == "3"
    assert cell_5.type == "float"
    assert cell_5.style == "ce5"
    assert cell_5.x == 5
    assert cell_5.y == 1


def test_get_cell_int(row):
    cell_5 = row.get_cell(5)
    assert cell_5.get_value() == 3
    assert cell_5.text_content == "3"
    assert cell_5.type == "float"
    assert cell_5.style == "ce5"


def test_get_cell_int_repr(row):
    cell_5 = row.get_cell(5)
    assert repr(cell_5) == "<Cell x=5 y=1>"


def test_get_cell_coord(row):
    cell = row.get_cell(-1)
    assert cell.get_value() == 3
    cell = row.get_cell(-3)
    assert cell.get_value() == 3
    cell = row.get_cell(-4)
    assert cell.get_value() == 2
    cell = row.get_cell(-5)
    assert cell.get_value() == 1
    cell = row.get_cell(-1 - 7)
    assert cell.get_value() == 3
    cell = row.get_cell(-3 - 56)
    assert cell.get_value() == 3
    cell = row.get_cell(-4 - 560)
    assert cell.get_value() == 2
    cell = row.get_cell(-5 - 7000)
    assert cell.get_value() == 1
    cell = row.get_cell(8)
    assert cell.get_value() is None
    cell = row.get_cell(1000)
    assert cell.get_value() is None


def test_get_value_coord(row):
    row.append_cell(Cell("Appended"))
    value = row.get_value(-1)
    assert value == "Appended"
    value = row.get_value(-3)
    assert value == 3
    value = row.get_value(-4)
    assert value == 3
    value = row.get_value(-5)
    assert value == 2
    value = row.get_value(-1 - 8)
    assert value == "Appended"
    value = row.get_value(7)
    assert value == "Appended"
    value = row.get_value(8)
    assert value is None
    value = row.get_value(1000)
    assert value is None


def test_get_value_coord_with_get_type(row):
    row.append_cell(Cell("Appended"))
    value = row.get_value(-1, get_type=True)
    assert value == ("Appended", "string")
    value = row.get_value(-3, get_type=True)
    assert value == (3, "float")
    value = row.get_value(-4, get_type=True)
    assert value == (3, "float")
    value = row.get_value(-5, get_type=True)
    assert value == (2, "float")
    value = row.get_value(-1 - 8, get_type=True)
    assert value == ("Appended", "string")
    value = row.get_value(7, get_type=True)
    assert value == ("Appended", "string")
    value = row.get_value(8, get_type=True)
    assert value == (None, None)
    value = row.get_value(1000, get_type=True)
    assert value == (None, None)


def test_set_cell(row):
    row.set_value(1, 3.14)
    assert row.get_values() == [1, dec("3.14"), 1, 2, 3, 3, 3]
    # Test repetitions are synchronized
    assert row.width == 7


def test_set_cell_far_away(row):
    row.set_value(7 + 3, 3.14)
    assert row.get_values() == [1, 1, 1, 2, 3, 3, 3, None, None, None, dec("3.14")]
    # Test repetitions are synchronized
    assert row.width == 11


def test_set_cell_repeat(row_repeats):
    row = row_repeats
    row.set_value(1, 3.14)
    assert row.get_values() == [1, dec("3.14"), 1, 2, 3, 3, 3]
    # Test repetitions are synchronized
    assert row.width == 7


def test_set_cell_repeat_repeat(row_repeats):
    row = row_repeats
    cell = Cell(value=20, repeated=2)
    row.set_cell(1, cell)
    assert row.get_values() == [1, 20, 20, 2, 3, 3, 3]
    # Test repetitions are synchronized
    assert row.width == 7


def test_set_cell_none(row_repeats):
    row = row_repeats
    row.set_cell(1, None)
    assert row.get_values() == [1, None, 1, 2, 3, 3, 3]
    # Test repetitions are synchronized
    assert row.width == 7


def test_insert(row):
    cell = row.insert_cell(3)
    assert type(cell) is Cell
    assert cell.x == 3
    assert cell.y == 1


def test_insert_cell(row):
    cell = row.insert_cell(3, Cell("Inserted"))
    assert row.width == 8
    assert row.get_values() == [1, 1, 1, "Inserted", 2, 3, 3, 3]
    # Test repetitions are synchronized
    assert row.width == 8
    assert cell.x == 3
    assert cell.y == 1


def test_insert_cell_repeat(row_repeats):
    row = row_repeats
    cell = row.insert_cell(6, Cell("Inserted"))
    assert row.get_values() == [1, 1, 1, 2, 3, 3, "Inserted", 3]
    # Test repetitions are synchronized
    assert row.width == 8
    assert cell.x == 6
    assert cell.y == 0


def test_insert_cell_width(row_repeats):
    row = row_repeats
    assert row.width == 7
    cell = row.insert_cell(7, Cell("Inserted"))
    assert row.get_values() == [1, 1, 1, 2, 3, 3, 3, "Inserted"]
    # Test repetitions are synchronized
    assert row.width == 8
    assert cell.x == 7
    assert cell.y == 0


def test_insert_cell_repeat_repeat(row_repeats):
    row = row_repeats
    cell = row.insert_cell(6, Cell("Inserted", repeated=3))
    assert row.get_values() == [
        1,
        1,
        1,
        2,
        3,
        3,
        "Inserted",
        "Inserted",
        "Inserted",
        3,
    ]
    # Test repetitions are synchronized
    assert row.width == 10
    assert cell.x == 6
    assert cell.y == 0


def test_insert_cell_repeat_repeat_bis(row_repeats):
    row = row_repeats
    cell = row.insert_cell(1, Cell("Inserted", repeated=2))
    assert row.get_values() == [1, "Inserted", "Inserted", 1, 1, 2, 3, 3, 3]
    # Test repetitions are synchronized
    assert row.width == 9
    assert cell.x == 1
    assert cell.y == 0


def test_extend_cells(row_repeats):
    row = row_repeats
    row.extend_cells()
    assert row.width == 7


def test_append_cell(row):
    orig_row = row.clone
    cell = row.append_cell()
    assert isinstance(cell, Cell)
    assert cell.x == orig_row.width
    assert cell.y == 1


def test_append_cell2(row):
    orig_row = row.clone
    cell = row.append_cell(Cell("Appended"))
    assert row.get_values() == [1, 1, 1, 2, 3, 3, 3, "Appended"]
    # Test repetitions are synchronized
    assert row.width == 8
    assert cell.x == orig_row.width
    assert cell.y == 1


def test_delete_cell(row):
    row.delete_cell(3)
    assert row.get_values() == [1, 1, 1, 3, 3, 3]
    # Test repetitions are synchronized
    assert row.width == 6


def test_delete_cell_width(row):
    row.delete_cell(7)
    assert row.get_values() == [1, 1, 1, 2, 3, 3, 3]
    assert row.width == 7


def test_delete_cell_width_2(row):
    row.delete_cell(8)
    assert row.get_values() == [1, 1, 1, 2, 3, 3, 3]
    assert row.width == 7


def test_delete_cell_repeat(row_repeats):
    row = row_repeats
    row.delete_cell(-1)
    assert row.get_values() == [1, 1, 1, 2, 3, 3]
    # Test repetitions are synchronized
    assert row.width == 6


def test_set_cells_1(row):
    cells = [Cell(value=10)]
    row.set_cells(cells)
    assert row.get_values() == [10, 1, 1, 2, 3, 3, 3]
    # Test repetitions are synchronized
    assert row.width == 7


def test_set_cells_1_start_none(row):
    cells = [Cell(value=10)]
    row.set_cells(cells, start=None)
    assert row.get_values() == [10, 1, 1, 2, 3, 3, 3]
    assert row.width == 7


def test_set_cells_none(row):
    cells = None
    row.set_cells(cells)
    assert row.get_values() == [1, 1, 1, 2, 3, 3, 3]
    assert row.width == 7


def test_set_cells_2(row):
    cells = [Cell(value=10), Cell(value=20)]
    row.set_cells(cells)
    assert row.get_values() == [10, 20, 1, 2, 3, 3, 3]
    # Test repetitions are synchronized
    assert row.width == 7


def test_set_cells_3(row):
    cells = [Cell(value=10), None, Cell(value=20)]
    row.set_cells(cells)
    assert row.get_values() == [10, None, 20, 2, 3, 3, 3]
    assert row.width == 7


def test_set_cells_many(row):
    cells = []
    for i in range(10):
        cells.append(Cell(value=10 * i))
    row.set_cells(cells)
    assert row.get_values() == [0, 10, 20, 30, 40, 50, 60, 70, 80, 90]
    # Test repetitions are synchronized
    assert row.width == 10


def test_set_cells_1_start_1(row):
    cells = [Cell(value=10)]
    row.set_cells(cells, 1)
    assert row.get_values() == [1, 10, 1, 2, 3, 3, 3]
    # Test repetitions are synchronized
    assert row.width == 7


def test_set_cells_1_start_m_2(row):
    cells = [Cell(value=10)]
    row.set_cells(cells, -2)
    assert row.get_values() == [1, 1, 1, 2, 3, 10, 3]
    # Test repetitions are synchronized
    assert row.width == 7


def test_set_cells_1_start_m_6(row):
    cells = [Cell(value=10)]
    row.set_cells(cells, 6)
    assert row.get_values() == [1, 1, 1, 2, 3, 3, 10]
    # Test repetitions are synchronized
    assert row.width == 7


def test_set_cells_1_start_m_9(row):
    cells = [Cell(value=10)]
    row.set_cells(cells, 9)
    assert row.get_values() == [1, 1, 1, 2, 3, 3, 3, None, None, 10]
    # Test repetitions are synchronized
    assert row.width == 10


def test_set_cells_2_start_1(row):
    cells = [Cell(value=10), Cell(value=20)]
    row.set_cells(cells, 1)
    assert row.get_values() == [1, 10, 20, 2, 3, 3, 3]
    # Test repetitions are synchronized
    assert row.width == 7


def test_set_cells_many_start_5(row):
    cells = []
    for i in range(5):
        cells.append(Cell(value=10 * i))
    row.set_cells(cells, 5)
    assert row.get_values() == [1, 1, 1, 2, 3, 0, 10, 20, 30, 40]
    # Test repetitions are synchronized
    assert row.width == 10


def test_set_cells_many_start_far(row):
    cells = []
    for i in range(5):
        cells.append(Cell(value=10 * i))
    row.set_cells(cells, 9)
    assert row.get_values() == [1, 1, 1, 2, 3, 3, 3, None, None, 0, 10, 20, 30, 40]
    # Test repetitions are synchronized
    assert row.width == 14


def test_set_cells_many_start_far_no_clone(row):
    cell_values = [Cell(value=i * 10) for i in range(12)]
    row.set_cells(cell_values, 0, clone=False)
    assert row.get_values() == [i * 10 for i in range(12)]
    # Test repetitions are synchronized
    assert row.width == 12


def test_set_cells_3_start_1_repeats(row):
    cells = [Cell(value=10, repeated=2)]
    row.set_cells(cells, 1)
    assert row.get_values() == [1, 10, 10, 2, 3, 3, 3]
    # Test repetitions are synchronized
    assert row.width == 7


def test_set_cells_3_start_1_repeats_2(row):
    cells = [Cell(value=10, repeated=2), Cell(value=20)]
    row.set_cells(cells, 1)
    assert row.get_values() == [1, 10, 10, 20, 3, 3, 3]
    # Test repetitions are synchronized
    assert row.width == 7


def test_set_cells_3_start_1_repeats_3(row):
    cells = [Cell(value=10, repeated=2), Cell(value=20), Cell(value=30, repeated=2)]
    row.set_cells(cells, 1)
    assert row.get_values() == [1, 10, 10, 20, 30, 30, 3]
    # Test repetitions are synchronized
    assert row.width == 7


def test_set_cells_3_start_1_repeats_4(row):
    cells = [Cell(value=10, repeated=2), Cell(value=20), Cell(value=30, repeated=4)]
    row.set_cells(cells, 1)
    assert row.get_values() == [1, 10, 10, 20, 30, 30, 30, 30]
    # Test repetitions are synchronized
    assert row.width == 8


def test_set_values_empty():
    row = Row()
    row.set_values([1, 2, 3, 4])
    assert row.width == 4
    assert row.get_values() == [1, 2, 3, 4]


def test_set_values_empty_start_none():
    row = Row()
    row.set_values([1, 2, 3, 4], start=None)
    assert row.width == 4
    assert row.get_values() == [1, 2, 3, 4]


def test_set_values_on_row(row):
    row.set_values([10, 20, 30, "4"])
    assert row.width == 7
    assert row.get_values() == [10, 20, 30, "4", 3, 3, 3]


def test_set_values_on_row2(row):
    row.set_values([10, 20, 30, "4"], start=2)
    assert row.width == 7
    assert row.get_values() == [1, 1, 10, 20, 30, "4", 3]


def test_set_values_on_row3(row):
    row.set_values([10, 20, 30, "4"], start=2)
    assert row.width == 7
    assert row.get_values() == [1, 1, 10, 20, 30, "4", 3]


def test_set_values_on_row4(row):
    row.set_values([10, 20, 30, "4"], start=-2)
    assert row.width == 9
    assert row.get_values() == [1, 1, 1, 2, 3, 10, 20, 30, "4"]


def test_set_values_on_row5(row):
    row.set_values([10, 20, 30, "4"], start=8)
    assert row.width == 7 + 1 + 4
    assert row.get_values() == [1, 1, 1, 2, 3, 3, 3, None, 10, 20, 30, "4"]


def test_set_values_on_row6(row):
    row.set_values([10, 20, 30, 40, 50, 60, 70, 80], start=0)
    assert row.width == 8
    assert row.get_values() == [10, 20, 30, 40, 50, 60, 70, 80]


def test_set_values_on_row_percentage(row):
    row.set_values([10, 20], start=4, cell_type="percentage")
    assert row.width == 7
    assert row.get_values() == [1, 1, 1, 2, 10, 20, 3]
    assert row.get_values(get_type=True, cell_type="percentage") == [
        (10, "percentage"),
        (20, "percentage"),
    ]


def test_set_values_on_row_style(row):
    row.set_values([10, 20], start=3, style="bold")
    assert row.width == 7
    assert row.get_values() == [1, 1, 1, 10, 20, 3, 3]
    assert row.get_cell(4).style == "bold"


def test_set_values_on_row_curency(row):
    row.set_values([10, 20], start=3, cell_type="currency", currency="EUR")
    assert row.width == 7
    assert row.get_values() == [1, 1, 1, 10, 20, 3, 3]
    assert row.get_cell(4).get_value(get_type=True) == (20, "currency")
    assert row.get_cell(4).currency == "EUR"



# Copyright 2018-2026 Jérôme Dumonteil
# Copyright (c) 2009-2010 Ars Aperta, Itaapy, Pierlis, Talend.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
#
# Authors (odfdo project): jerome.dumonteil@gmail.com
# The odfdo project is a derivative work of the lpod-python project:
# https://github.com/lpod/lpod-python
# Authors: Hervé Cauwelier <herve@itaapy.com>

from collections.abc import Iterable

import pytest

from odfdo.style import Style


@pytest.fixture
def style() -> Iterable[Style]:
    yield Style("paragraph")


def test_bad_family():
    style = Style("master-page")
    with pytest.raises(AttributeError):
        _x = style.set_background


def test_color(style):
    style.set_background(color="#abcdef")
    expected = (
        '<style:style style:family="paragraph">'
        "<style:paragraph-properties "
        'fo:background-color="#abcdef"/>'
        "</style:style>"
    )
    assert style.serialize() == expected


def test_image(style):
    style.set_background(url="Pictures/toto")
    # expected = (
    #     '<style:style style:family="paragraph">'
    #     "<style:paragraph-properties "
    #     'fo:background-color="transparent">'
    #     "<style:background-image "
    #     'xlink:href="Pictures/toto" '
    #     'style:position="center"/>'
    #     "</style:paragraph-properties>"
    #     "</style:style>"
    # )
    expected = (
        '<style:style style:family="paragraph">'
        "<style:paragraph-properties>"
        "<style:background-image "
        'xlink:href="Pictures/toto" '
        'style:position="center"/>'
        "</style:paragraph-properties>"
        "</style:style>"
    )
    assert style.serialize() == expected


def test_image_full(style):
    style.set_background(
        url="Pictures/toto",
        position="top left",
        repeat="no-repeat",
        opacity=50,
        filter="myfilter",
    )
    # expected = (
    #     '<style:style style:family="paragraph">'
    #     "<style:paragraph-properties "
    #     'fo:background-color="transparent">'
    #     "<style:background-image "
    #     'xlink:href="Pictures/toto" '
    #     'style:position="top left" '
    #     'style:repeat="no-repeat" '
    #     'draw:opacity="50" '
    #     'style:filter-name="myfilter"/>'
    #     "</style:paragraph-properties>"
    #     "</style:style>"
    # )
    expected = (
        '<style:style style:family="paragraph">'
        "<style:paragraph-properties>"
        "<style:background-image "
        'xlink:href="Pictures/toto" '
        'style:position="top left" '
        'style:repeat="no-repeat" '
        'draw:opacity="50" '
        'style:filter-name="myfilter"/>'
        "</style:paragraph-properties>"
        "</style:style>"
    )
    assert style.serialize() == expected



#!/usr/bin/env python
"""Scan a list of files and directories (recursion), open all ODF documents
and copy document images to a target directory.
"""

import os
import sys
import time
from hashlib import sha256
from pathlib import Path

from odfdo import Document

_DOC_SEQUENCE = 530
OUTPUT_DIR = Path(__file__).parent / "recipes_output" / "collected_pics"
DATA = Path(__file__).parent / "data"

# encoding = "UTF8"
known_images = set()
counter_image = 0
counter_odf = 0
counter_outside = 0


def store_image(path: Path, name: str, content: bytes) -> None:
    """Save the found image in result directory.

    Image new name is "odffile_imagename"."""
    global counter_image

    base = path.name.replace(".", "_")
    cpt = 1
    if not OUTPUT_DIR.is_dir():
        OUTPUT_DIR.mkdir(parents=True, exist_ok=True)
    target = OUTPUT_DIR / f"{base}_{cpt}_{name}"
    while target.exists():
        cpt += 1
        target = OUTPUT_DIR / f"{base}_{cpt}_{name}"
    target.write_bytes(content)
    counter_image += 1


def parse_odf_pics(path: Path) -> None:
    """Using odfdo for:
    - open possible ODF document: Document (including URI)
    - find images inside the document: get_image_list, get_attribute
    """
    if not path.suffix.lower().startswith(".od"):
        return
    try:
        document = Document(path)
    except Exception:
        return

    global counter_odf
    global counter_outside

    counter_odf += 1
    for image in document.body.images:
        image_url = image.url
        if not image_url:
            continue
        try:
            image_content = document.get_part(image_url)
        except KeyError:
            print("- not found inside document:", path)
            print("  image URL:", image_url)
            counter_outside += 1
            continue
        image_name = image_url.split("/")[-1]
        if not known_pic(image_content):
            store_image(path, image_name, image_content)


def known_pic(content: bytes) -> bool:
    """Remember the images already seen by sha256 fingerprint."""
    fingerprint = sha256(content).digest()
    if fingerprint in known_images:
        return True
    known_images.add(fingerprint)
    return False


def scan_document(source: Path) -> list[int]:
    """Recursively parse ODF files to copy images."""
    t0 = time.time()
    for path in source.glob("**/*"):
        if path.is_file():
            parse_odf_pics(path)
    elapsed = time.time() - t0
    print(
        f"{counter_image} images copied ({counter_outside} not found) from "
        f"{counter_odf} ODF files to {OUTPUT_DIR} in {elapsed:.2f}sec."
    )
    return [counter_image, counter_outside, counter_odf]


def main() -> None:
    try:
        source = Path(sys.argv[1])
    except IndexError:
        source = DATA

    result = scan_document(source)
    test_unit(result)


def test_unit(result: list[int]) -> None:
    # only for test suite:
    if "ODFDO_TESTING" not in os.environ:
        return

    assert result[0] == 15
    assert result[1] == 0
    assert result[2] == 19


if __name__ == "__main__":
    main()



# Copyright 2018-2026 Jérôme Dumonteil
# Copyright (c) 2009-2013 Ars Aperta, Itaapy, Pierlis, Talend.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
#
# Authors (odfdo project): jerome.dumonteil@gmail.com
# The odfdo project is a derivative work of the lpod-python project:
# https://github.com/lpod/lpod-python
"""Mixin class for "text:a" tag."""

from __future__ import annotations

from typing import TYPE_CHECKING

from .element import Element

if TYPE_CHECKING:
    from .link import Link


class LinkMixin(Element):
    """Mixin class for elements that can contain links.

    Used by the following classes:
         - "text:h"
         - "text:meta"
         - "text:meta-field"
         - "text:p"
         - "text:ruby-base"
         - "text:span"

    Also used for convenience by "office:text", "office:spreadsheet",
    "office:presentation", "text:note-body", "text:note", "text:section",
    "office:annotation".
    """

    def get_links(
        self,
        name: str | None = None,
        title: str | None = None,
        url: str | None = None,
        content: str | None = None,
    ) -> list[Link]:
        """Returns all links that match the specified criteria.

        Args:
            name: The name of the link.
            title: The title of the link.
            url: A regex pattern to match against the link's URL.
            content: A regex pattern to match against the link's content.

        Returns:
            list[Link]: A list of Link instances matching the criteria.
        """
        return self._filtered_elements(
            "descendant::text:a",
            office_name=name,
            office_title=title,
            url=url,
            content=content,
        )  # type: ignore [return-value]

    def get_link(
        self,
        position: int = 0,
        name: str | None = None,
        title: str | None = None,
        url: str | None = None,
        content: str | None = None,
    ) -> Link | None:
        """Returns a single link that matches the specified criteria.

        Args:
            position: The 0-based index of the matching link to return.
            name: The name of the link.
            title: The title of the link.
            url: A regex pattern to match against the link's URL.
            content: A regex pattern to match against the link's content.

        Returns:
            Link | None: A Link instance, or None if no link matches the criteria.
        """
        return self._filtered_element(
            "descendant::text:a",
            position,
            office_name=name,
            office_title=title,
            url=url,
            content=content,
        )  # type: ignore [return-value]



"""Minimal example of how to add a paragraph."""

from odfdo import Document, Paragraph

_DOC_SEQUENCE = 12


def main() -> None:
    document = Document("text")
    body = document.body

    # create a new paragraph with some content :
    paragraph = Paragraph("Hello World")
    body.append(paragraph)


if __name__ == "__main__":
    main()



#!/usr/bin/env python
"""Minimal example of how to add a Header of first level to a text document."""

import os

from odfdo import Document, Header

_DOC_SEQUENCE = 67


def generate_document() -> Document:
    """Return a document with a title."""
    document = Document("text")

    title1 = Header(1, "The Title")
    document.body.append(title1)

    return document


def main() -> None:
    document = generate_document()
    test_unit(document)


def test_unit(document: Document) -> None:
    # only for test suite:
    if "ODFDO_TESTING" not in os.environ:
        return

    title = document.body.get_header(0)
    assert str(title).strip() == "The Title"


if __name__ == "__main__":
    main()



# Copyright 2018-2026 Jérôme Dumonteil
# Copyright (c) 2009-2010 Ars Aperta, Itaapy, Pierlis, Talend.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
#
# Authors (odfdo project): jerome.dumonteil@gmail.com
# The odfdo project is a derivative work of the lpod-python project:
# https://github.com/lpod/lpod-python

import odfdo.const as const
from odfdo.const import OFFICE_PREFIX, USE_LO_EXTENSIONS, _generate_office_prefix


def test_use_lo_extensions():
    "By default USE_LO_EXTENSIONS is True"
    assert USE_LO_EXTENSIONS


def test_use_office_prefix():
    tags = OFFICE_PREFIX.decode().strip().split("\n")
    assert len(tags) == 49


def test_use_office_prefix_1():
    prefix = OFFICE_PREFIX.decode()
    assert "xmlns:office=" in prefix


def test_use_office_prefix_2():
    prefix = OFFICE_PREFIX.decode()
    assert "xmlns:ooo=" in prefix


def test_use_office_prefix_3():
    prefix = OFFICE_PREFIX.decode()
    assert "xmlns:loext=" in prefix


def test_use_lo_extensions_false():
    const.USE_LO_EXTENSIONS = False
    prefix = _generate_office_prefix()
    tags = prefix.decode().strip().split("\n")
    assert len(tags) == 34


def test_use_lo_extensions_false_1():
    const.USE_LO_EXTENSIONS = False
    prefix = _generate_office_prefix().decode()
    assert "xmlns:loext=" not in prefix



#!/usr/bin/env python
"""Read a document from BytesIO."""

import io
import os
from pathlib import Path

from odfdo import Document

_DOC_SEQUENCE = 600
DATA = Path(__file__).parent / "data"
SOURCE = "lorem.odt"


def document_from_bytesio() -> Document:
    file_path = DATA / SOURCE
    with io.BytesIO() as bytes_content:
        # read the file in the BytesIO (or read from some network)
        bytes_content.write(file_path.read_bytes())
        # Create the odfdo.Document from the BytesIO
        bytes_content.seek(0)
        document = Document(bytes_content)
        return document


def main() -> None:
    document = document_from_bytesio()
    test_unit(document)


def test_unit(document: Document) -> None:
    # only for test suite:
    if "ODFDO_TESTING" not in os.environ:
        return

    assert document.body.search("Lorem ipsum dolor sit amet") is not None


if __name__ == "__main__":
    main()



# Copyright 2018-2026 Jérôme Dumonteil
# Copyright (c) 2009-2010 Ars Aperta, Itaapy, Pierlis, Talend.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
#
# Authors (odfdo project): jerome.dumonteil@gmail.com
# The odfdo project is a derivative work of the lpod-python project:
# https://github.com/lpod/lpod-python
# Authors: Hervé Cauwelier <herve@itaapy.com>
#          David Versmisse <david.versmisse@itaapy.com>
"""Command-line script to display the content of an ODF document.

This script dumps the text content, styles, and metadata of an ODF file
to the standard output or into a specified directory.
"""

from __future__ import annotations

import sys
from argparse import ArgumentParser, Namespace
from pathlib import Path
from shutil import rmtree

from odfdo import Document, __version__

PROG = "odfdo-show"


def configure_parser() -> ArgumentParser:
    description = (
        "Display various parts of an ODF document, including text content, "
        "styles, and metadata, to standard output or a specified directory."
    )
    epilog = (
        "By default, the script dumps the main textual content. "
        "Use options like -m (metadata), -s (styles), or -o (output directory) "
        "to customize the output. When using -o, embedded pictures are also extracted."
    )
    parser = ArgumentParser(prog=PROG, description=description, epilog=epilog)

    parser.add_argument(
        "--version",
        action="version",
        version=f"{PROG} v{__version__}",
    )
    parser.add_argument(
        "-m",
        "--meta",
        action="store_true",
        default=False,
        help="dump metadata to stdout",
    )
    parser.add_argument(
        "-s",
        "--styles",
        action="store_true",
        default=False,
        help="dump styles to stdout",
    )
    parser.add_argument(
        "-n",
        "--no-content",
        action="store_true",
        default=False,
        help="do not dump content to stdout",
    )
    parser.add_argument(
        "-r",
        "--rst",
        action="store_true",
        default=False,
        help="dump the content file with a reST syntax",
    )
    parser.add_argument(
        "-o",
        "--output",
        metavar="DIR",
        help="dump the output into DIR",
    )
    parser.add_argument(
        "input",
        action="store",
        help="input ODF file to show",
    )
    return parser


def parse_cli_args(cli_args: list[str] | None = None) -> Namespace:
    parser = configure_parser()
    return parser.parse_args(cli_args)


def clean_filename(name: str) -> str:
    allowed_characters = {".", "-", "@"}
    result = []
    for char in name:
        if char in allowed_characters or char.isalnum():
            result.append(char)
        else:
            result.append("\t")
    return "_".join("".join(result).split())


def dump_pictures(document: Document, target: str | Path) -> None:
    for part_name in document.parts:
        if not part_name.startswith("Pictures/"):
            continue
        path = Path(target, "Pictures")
        if not path.exists():
            path.mkdir(parents=False, exist_ok=False)
        data = document.get_part(part_name)
        path = Path(target, part_name)
        path.write_bytes(data)  # type: ignore


def spreadsheet_to_stdout(document: Document) -> None:
    body = document.body
    for table in body.tables:
        table.rstrip(aggressive=True)
        print(table.to_csv(None))


def spreadsheet_to_csv(document: Document, output: Path) -> None:
    body = document.body
    for table in body.tables:
        name = str(table.name)
        filename = clean_filename(name) + ".csv"
        print(filename)
        table.rstrip(aggressive=True)
        table.to_csv(output / filename)


def print_format_error(doc_type: str) -> None:
    msg = f"Error: The OpenDocument format '{doc_type}' is not supported yet."
    print(msg, file=sys.stderr)


def check_target_directory(path: Path) -> None:
    if path.exists():  # pragma: no cover
        message = f'The path "{path}" exists, overwrite it? [y/n]'
        print(message, file=sys.stderr)
        line = sys.stdin.readline()
        line = line.strip().lower()
        if line != "y":
            print("Operation aborted", file=sys.stderr)
            raise SystemExit(0)


def show_output(
    args: Namespace,
    doc: Document,
    doc_type: str,
) -> None:
    output = Path(args.output)
    check_target_directory(output)
    if output.exists():  # pragma: no cover
        rmtree(output)
    output.mkdir(parents=True, exist_ok=True)
    (output / "meta.txt").write_text(doc.get_formated_meta())
    (output / "styles.txt").write_text(doc.show_styles())
    dump_pictures(doc, output)

    if doc_type in {"text", "text-template", "presentation", "presentation-template"}:
        (output / "content.rst").write_text(doc.get_formatted_text(rst_mode=args.rst))
    elif doc_type in {"spreadsheet", "spreadsheet-template"}:
        spreadsheet_to_csv(doc, output)
    else:
        print_format_error(doc_type)
        raise SystemExit(1)


def show(args: Namespace) -> None:
    doc = Document(args.input)
    doc_type = doc.get_type()

    if args.output:
        return show_output(args, doc, doc_type)
    if args.meta:
        print(doc.get_formated_meta())
    if args.styles:
        print(doc.show_styles())
    if doc_type in {"text", "text-template", "presentation", "presentation-template"}:
        if not args.no_content:
            print(doc.get_formatted_text(rst_mode=args.rst))
    elif doc_type in {"spreadsheet", "spreadsheet-template"}:
        if not args.no_content:
            spreadsheet_to_stdout(doc)
    else:
        print_format_error(doc_type)
        raise SystemExit(1)


def main() -> None:
    args: Namespace = parse_cli_args()
    main_show(args)


def main_show(args: Namespace) -> None:
    try:
        show(args)
    except Exception as e:
        configure_parser().print_help()
        print()
        print(f"Error: {e.__class__.__name__}, {e}")
        raise SystemExit(1) from None


if __name__ == "__main__":
    main()



#!/usr/bin/env python
"""Remove all the links (the text:a tag), keeping the inner text."""

import os
import sys
from pathlib import Path

from odfdo import Document, Link, remove_tree

_DOC_SEQUENCE = 510
DATA = Path(__file__).parent / "data"
SOURCE = "collection2.odt"
OUTPUT_DIR = Path(__file__).parent / "recipes_output" / "nolink"
TARGET = "document.odt"


def read_source_document() -> Document:
    """Return the source Document."""
    try:
        source = sys.argv[1]
    except IndexError:
        source = DATA / SOURCE
    return Document(source)


def save_new(document: Document, name: str) -> None:
    """Save a recipe result Document."""
    OUTPUT_DIR.mkdir(parents=True, exist_ok=True)
    new_path = OUTPUT_DIR / name
    print("Saving:", new_path)
    document.save(new_path, pretty=True)


def remove_all_links(document: Document) -> list[tuple[str, int]]:
    """Remove all links and return statistics."""
    body = document.body
    result: list[tuple[str, int]] = []

    result.append(("source, links occurrences:", len(body.get_links())))
    remove_tree(body, Link)
    result.append(("final, links occurrences:", len(body.get_links())))

    for line in result:
        print(line[0], line[1])

    return result


def main() -> None:
    document = read_source_document()
    result = remove_all_links(document)
    test_unit(result)
    save_new(document, TARGET)


def test_unit(result: list[tuple[str, int]]) -> None:
    # only for test suite:
    if "ODFDO_TESTING" not in os.environ:
        return

    assert result[0][1] == 352
    assert result[1][1] == 0


if __name__ == "__main__":
    main()



#!/usr/bin/env python
"""Create a minimal spreadsheet with "Hello World" in the first cell."""

import os
from pathlib import Path

from odfdo import Document, Table

_DOC_SEQUENCE = 5
OUTPUT_DIR = Path(__file__).parent / "recipes_output" / "basic_ods"
TARGET = "document.ods"


def save_new(document: Document, name: str) -> None:
    """Save a recipe result Document."""
    OUTPUT_DIR.mkdir(parents=True, exist_ok=True)
    new_path = OUTPUT_DIR / name
    print("Saving:", new_path)
    document.save(new_path, pretty=True)


def generate_document() -> Document:
    document = Document("spreadsheet")
    body = document.body
    body.clear()

    table = Table("Hello table")
    table.set_value("A1", "Hello World")
    body.append(table)

    return document


def main() -> None:
    document = generate_document()
    test_unit(document)
    save_new(document, TARGET)


def test_unit(document: Document) -> None:
    # only for test suite:
    if "ODFDO_TESTING" not in os.environ:
        return

    text = document.body.get_table(0).get_cell((0, 0)).value.strip()
    print(text)
    assert text == "Hello World"


if __name__ == "__main__":
    main()



# Copyright 2018-2026 Jérôme Dumonteil
# Copyright (c) 2009-2010 Ars Aperta, Itaapy, Pierlis, Talend.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
#
# Authors (odfdo project): jerome.dumonteil@gmail.com
# The odfdo project is a derivative work of the lpod-python project:
# https://github.com/lpod/lpod-python
# Authors: Hervé Cauwelier <herve@itaapy.com>

from collections.abc import Iterable

import pytest

from odfdo.style import Style


@pytest.fixture
def style() -> Iterable[Style]:
    yield Style("list")


def test_get_level_style(style):
    level_style = style.get_level_style(1)
    assert level_style is None


def test_set_level_style(style):
    style.set_level_style(1, num_format="1")
    level_style = style.get_level_style(1)
    assert level_style is not None


def test_set_level_style_number_missing_format(style):
    with pytest.raises(ValueError):
        style.set_level_style(1)


def test_set_level_style_number(style):
    level_style = style.set_level_style(1, num_format="1")
    assert isinstance(level_style, Style)
    expected = '<text:list-level-style-number text:level="1" fo:num-format="1"/>'
    assert level_style.serialize() == expected


def test_set_level_style_bullet(style):
    level_style = style.set_level_style(2, bullet_char="·")
    assert isinstance(level_style, Style)
    expected = '<text:list-level-style-bullet text:level="2" text:bullet-char="·"/>'
    assert level_style.serialize() == expected


def test_set_level_style_image(style):
    level_style = style.set_level_style(3, url="bullet.png")
    assert isinstance(level_style, Style)
    expected = '<text:list-level-style-image text:level="3" xlink:href="bullet.png"/>'
    assert level_style.serialize() == expected


def test_set_level_style_full(style):
    level_style = style.set_level_style(
        3,
        num_format="1",
        prefix=" ",
        suffix=".",
        display_levels=3,
        start_value=2,
        style="MyList",
    )
    expected = (
        "<text:list-level-style-number "
        'text:level="3" fo:num-format="1" '
        'style:num-prefix=" " style:num-suffix="." '
        'text:display-levels="3" text:start-value="2" '
        'text:style-name="MyList"/>'
    )
    assert level_style.serialize() == expected


def test_set_level_style_clone(style):
    level_1 = style.set_level_style(1, num_format="1")
    level_2 = style.set_level_style(2, display_levels=2, clone=level_1)
    expected = (
        "<text:list-level-style-number "
        'text:level="2" fo:num-format="1" '
        'text:display-levels="2"/>'
    )
    assert level_2.serialize() == expected



# Copyright 2018-2026 Jérôme Dumonteil
# Copyright (c) 2009-2013 Ars Aperta, Itaapy, Pierlis, Talend.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
#
# Authors (odfdo project): jerome.dumonteil@gmail.com
# The odfdo project is a derivative work of the lpod-python project:
# https://github.com/lpod/lpod-python
# Authors: Jerome Dumonteil <jerome.dumonteil@itaapy.com>
"""Command-line script to convert ODF files to/from a folder structure.

This script allows for converting a standard ODF file (which is a zip archive)
into a directory containing its component XML files and assets, and vice-versa.
This is useful for version control and manual inspection of ODF documents.
"""

from __future__ import annotations

from argparse import ArgumentParser, Namespace
from pathlib import Path

from odfdo import Document, __version__
from odfdo.const import FOLDER, ZIP

PROG = "odfdo-folder"


def configure_parser() -> ArgumentParser:
    description = (
        "Convert a standard ODF file (zip archive) to a folder "
        "structure, or convert a folder structure back to an ODF file."
    )
    epilog = (
        "This tool is useful for version control and manual inspection "
        "of ODF documents, as it allows access to the component XML files "
        "and assets within the ODF package."
    )
    parser = ArgumentParser(prog=PROG, description=description, epilog=epilog)
    parser.add_argument(
        "--version",
        action="version",
        version=f"{PROG} v{__version__}",
    )
    parser.add_argument(
        "file_or_folder",
        action="store",
        help="file or folder to convert",
    )
    return parser


def parse_cli_args(cli_args: list[str] | None = None) -> Namespace:
    parser = configure_parser()
    return parser.parse_args(cli_args)


def main() -> None:
    args: Namespace = parse_cli_args()
    main_convert_folder(args)


def main_convert_folder(args: Namespace) -> None:
    try:
        convert_folder(args.file_or_folder)
    except Exception as e:
        configure_parser().print_help()
        print()
        print(f"Error: {e}")
        raise SystemExit(1) from None


def convert_folder(path_str: str) -> None:
    path = Path(path_str)
    if path.is_file():
        out_packaging = FOLDER
    elif path.is_dir():
        out_packaging = ZIP
    else:
        raise ValueError(f"Not a file or folder: {path}")
    document = Document(path)
    document.save(packaging=out_packaging, pretty=True)


if __name__ == "__main__":
    main()



# Copyright 2018-2026 Jérôme Dumonteil
# Copyright (c) 2009-2010 Ars Aperta, Itaapy, Pierlis, Talend.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
#
# Authors (odfdo project): jerome.dumonteil@gmail.com
# The odfdo project is a derivative work of the lpod-python project:
# https://github.com/lpod/lpod-python

from __future__ import annotations

from collections.abc import Iterable

import pytest

from odfdo import Element
from odfdo.shapes import RegularPolygonShape


@pytest.fixture
def poly() -> Iterable[RegularPolygonShape]:
    shape = RegularPolygonShape(
        name="Name",
        corners=4,
        concave=False,
        size=("10cm", "10cm"),
        position=("3cm", "4cm"),
    )
    yield shape


@pytest.fixture
def polyconcave() -> Iterable[RegularPolygonShape]:
    shape = RegularPolygonShape(
        name="Name",
        corners=4,
        concave=True,
        sharpness="1cm",
        size=("10cm", "10cm"),
        position=("3cm", "4cm"),
    )
    yield shape


def test_regular_poly_minimal():
    shape = RegularPolygonShape()
    assert shape._canonicalize() == "<draw:regular-polygon></draw:regular-polygon>"


def test_regular_poly_class():
    shape = Element.from_tag("<draw:regular-polygon/>")
    assert isinstance(shape, RegularPolygonShape)


def test_regular_poly_create(poly):
    assert poly._canonicalize() == (
        '<draw:regular-polygon draw:corners="4" '
        'draw:name="Name" '
        'svg:height="10cm" '
        'svg:width="10cm" '
        'svg:x="3cm" '
        'svg:y="4cm">'
        "</draw:regular-polygon>"
    )


def test_regular_poly_corners(poly):
    assert poly.corners == 4


def test_regular_poly_corners_none(poly):
    # shout never happen
    poly.corners = None
    assert poly.corners is None


def test_regular_poly_concave_1(polyconcave):
    assert polyconcave.concave is True


def test_regular_poly_concave_2(polyconcave):
    assert polyconcave.sharpness == "1cm"


def test_regular_poly_position_1(poly):
    assert poly.position == ("3cm", "4cm")


def test_regular_poly_position_2(poly):
    assert poly.pos_x == "3cm"


def test_regular_poly_position_3(poly):
    assert poly.pos_y == "4cm"


def test_regular_poly_size_1(poly):
    assert poly.size == ("10cm", "10cm")


def test_regular_poly_size_2(poly):
    assert poly.height == "10cm"


def test_regular_poly_size_3(poly):
    assert poly.width == "10cm"



# Copyright 2018-2026 Jérôme Dumonteil
# Copyright (c) 2009-2013 Ars Aperta, Itaapy, Pierlis, Talend.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
#
# Authors (odfdo project): jerome.dumonteil@gmail.com
# The odfdo project is a derivative work of the lpod-python project:
# https://github.com/lpod/lpod-python
"""Command-line script to convert ODF files to/from flat XML format.

This script allows for converting a standard ODF file (which is a zip archive)
or a folder structure into a flat ODF XML file (plain XML format), and vice-versa.
This is useful for version control, manual inspection, and text processing of
ODF documents.
"""

from __future__ import annotations

from argparse import ArgumentParser, Namespace
from pathlib import Path

from odfdo import Document, __version__
from odfdo.const import XML, ZIP

PROG = "odfdo-flat"

FLAT_EXT_MAP: dict[str, str] = {
    ".fodt": ".odt",
    ".fods": ".ods",
    ".fodp": ".odp",
    ".fodg": ".odg",
    ".fodc": ".odc",
    ".fodi": ".odi",
    ".fodf": ".odf",
}


def configure_parser() -> ArgumentParser:
    description = (
        "Convert a standard ODF file (zip archive) or folder structure "
        "to a flat ODF XML file, or convert a flat XML file back to an ODF file."
    )
    epilog = (
        "This tool is useful for version control and manual inspection "
        "of ODF documents, as the flat XML format is a single plain text file "
        "that can be easily viewed, edited, and compared with standard text tools. "
        "Flat ODF files typically have extensions like .fodt, .fods, .fodp, .fodg, etc."
    )
    parser = ArgumentParser(prog=PROG, description=description, epilog=epilog)
    parser.add_argument(
        "--version",
        action="version",
        version=f"{PROG} v{__version__}",
    )
    parser.add_argument(
        "file_or_folder",
        action="store",
        help="file or folder to convert",
    )
    return parser


def parse_cli_args(cli_args: list[str] | None = None) -> Namespace:
    parser = configure_parser()
    return parser.parse_args(cli_args)


def main() -> None:
    args: Namespace = parse_cli_args()
    main_convert_flat(args)


def main_convert_flat(args: Namespace) -> None:
    try:
        convert_flat(args.file_or_folder)
    except Exception as e:
        configure_parser().print_help()
        print()
        print(f"Error: {e}")
        raise SystemExit(1) from None


def is_flat_xml_file(path: Path) -> bool:
    """Check if the path is a flat ODF XML file by extension or content."""
    # Check extension first
    if path.suffix.lower() in FLAT_EXT_MAP:
        return True
    # If .xml extension or no recognized extension, check content
    if path.suffix.lower() in (".xml", ""):
        try:
            content = path.read_bytes()
            # Quick check for XML declaration and office:document element
            if content.lstrip().startswith(b"<?xml") and b"office:document" in content:
                # Just bet format is ok
                return True
        except OSError:
            pass
    return False


def convert_flat(path_str: str) -> None:
    path = Path(path_str)

    if not path.exists():
        raise ValueError(f"Path does not exist: {path}")

    # Determine if input is flat XML or needs to be converted to flat XML
    if path.is_dir():
        # Folder -> Flat XML
        out_packaging = XML
        document = Document(path)
        document.save(packaging=out_packaging, pretty=True)
    elif path.is_file():
        if is_flat_xml_file(path):
            # Flat XML -> ZIP (standard ODF)
            out_packaging = ZIP
            document = Document(path)
            # Change extension from flat to regular ODF
            # e.g., .fodt -> .odt, .fods -> .ods
            flat_ext = path.suffix.lower()
            new_ext = FLAT_EXT_MAP.get(flat_ext, ".odt")
            target_path = path.with_suffix(new_ext)
            document.save(
                target=target_path,
                packaging=out_packaging,
                pretty=True,
            )
        else:
            # ZIP or other ODF file -> Flat XML
            out_packaging = XML
            document = Document(path)
            document.save(packaging=out_packaging, pretty=True)
    else:
        raise ValueError(f"Not a file or folder: {path}")


if __name__ == "__main__":
    main()



# Copyright 2018-2026 Jérôme Dumonteil
# Copyright (c) 2009-2013 Ars Aperta, Itaapy, Pierlis, Talend.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
#
# Authors (odfdo project): jerome.dumonteil@gmail.com
# The odfdo project is a derivative work of the lpod-python project:
# https://github.com/lpod/lpod-python
"""Command-line script to convert an ODF text document to Markdown.

This script reads an ODF text document and prints its content to standard
output in Markdown format.
"""

from __future__ import annotations

from argparse import ArgumentParser, Namespace

from odfdo import __version__
from odfdo.utils.script_utils import read_document

PROG = "odfdo-markdown"


def configure_parser() -> ArgumentParser:
    description = (
        "Convert an ODF text document to Markdown format and print to standard output."
    )
    epilog = (
        "This tool is useful for extracting the textual content of an ODF "
        "document in a lightweight, human-readable, and version-control-friendly "
        "format. It processes the document's main body content."
    )
    parser = ArgumentParser(prog=PROG, description=description, epilog=epilog)
    parser.add_argument(
        "--version",
        action="version",
        version=f"{PROG} v{__version__}",
    )
    parser.add_argument(
        "document",
        nargs="?",
        default=None,
        action="store",
        help="input document. if not present, read from stdin",
    )
    return parser


def parse_cli_args(cli_args: list[str] | None = None) -> Namespace:
    parser = configure_parser()
    return parser.parse_args(cli_args)


def to_md(args: Namespace) -> None:
    document = read_document(args.document)
    print(document.to_markdown())


def main() -> int:
    args: Namespace = parse_cli_args()
    return main_to_md(args)


def main_to_md(args: Namespace) -> int:
    try:
        to_md(args)
    except Exception:
        configure_parser().print_help()
        print()
        raise
    return 0


if __name__ == "__main__":
    raise SystemExit(main())



# Copyright 2018-2026 Jérôme Dumonteil
# Copyright (c) 2009-2013 Ars Aperta, Itaapy, Pierlis, Talend.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
#
# Authors (odfdo project): jerome.dumonteil@gmail.com
# The odfdo project is a derivative work of the lpod-python project:
# https://github.com/lpod/lpod-python
# Authors: David Versmisse <david.versmisse@itaapy.com>
#          Hervé Cauwelier <herve@itaapy.com>
#          Romain Gauthier <romain@itaapy.com>
#          Jerome Dumonteil <jerome.dumonteil@itaapy.com>
"""Spacer class for "text:s" tag."""

from __future__ import annotations

from typing import Any

from .element import Element, PropDef, PropDefBool, register_element_class
from .mixin_md import MDSpacer


class Spacer(MDSpacer, Element):
    """Representation of several spaces, "text:s".

    This element shall be used to represent the second and all following " "
    (U+0020, SPACE) characters in a sequence of " " (U+0020, SPACE) characters.
    It's good practice to use this element only for the second and all
    following SPACE characters in a sequence, though it's not an error if
    the preceding character is not a white space.
    """

    _tag = "text:s"
    _properties: tuple[PropDef | PropDefBool, ...] = (PropDef("number", "text:c"),)

    def __init__(self, number: int | None = 1, **kwargs: Any) -> None:
        """Create a Spacer element, "text:s", representing several spaces.

        Args:
            number: The number of spaces. Defaults to 1.
        """
        super().__init__(**kwargs)
        if self._do_init:
            if number and number >= 2:
                self.number: str | None = str(number)
            else:
                self.number = None

    def __str__(self) -> str:
        return self.text

    @property
    def text(self) -> str:
        """Get the string representation of the spacer.

        Returns:
            str: A string composed of spaces, e.g., "   ".
        """
        return " " * self.length

    @text.setter
    def text(self, text: str | None) -> None:
        """Set the string value of the spacer, which updates its length.

        Args:
            text: The string to set.
        """
        if text is None:
            text = ""
        self.length = len(text)

    @property
    def length(self) -> int:
        """Get the number of spaces represented by the spacer.

        Returns:
            int: The number of spaces.
        """
        value = self._base_attrib_getter("text:c")
        if value is None:
            return 1  # minimum 1 space
        return int(value)

    @length.setter
    def length(self, value: int | None) -> None:
        """Set the number of spaces represented by the spacer.

        Args:
            value: The number of spaces to set. If None or less
                than 2, the `text:c` attribute is removed, defaulting to 1 space.
        """
        if value is None or int(value) < 2:
            self._base_attrib_setter("text:c", None)
        else:
            self._base_attrib_setter("text:c", int(value))


Spacer._define_attribut_property()

register_element_class(Spacer)



# Copyright 2018-2026 Jérôme Dumonteil
# Copyright (c) 2009-2010 Ars Aperta, Itaapy, Pierlis, Talend.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
#
# Authors (odfdo project): jerome.dumonteil@gmail.com
# The odfdo project is a derivative work of the lpod-python project:
# https://github.com/lpod/lpod-python

from collections.abc import Iterable

import pytest

from odfdo.document import Document
from odfdo.row import Row
from odfdo.row_group import RowGroup
from odfdo.table import Table


@pytest.fixture
def table(samples) -> Iterable[Table]:
    # simple table with row group:
    #
    #   a2	b2
    #   a3	b3
    document = Document(samples("rowgroup.ods"))
    yield document.body.get_table(0)


def test_create_minimal_rg():
    rg = RowGroup()
    result = "<table:table-row-group/>"
    assert rg.serialize() == result


def test_create_rg_1():
    rg = RowGroup(height=1)
    result = "<table:table-row-group><table:table-row/></table:table-row-group>"
    assert rg.serialize() == result


def test_create_repr():
    rg = RowGroup(height=1)
    assert repr(rg) == "<RowGroup>"


def test_create_rg_2():
    rg = RowGroup(height=2)
    result = (
        "<table:table-row-group>"
        "<table:table-row/><table:table-row/>"
        "</table:table-row-group>"
    )
    assert rg.serialize() == result


def test_create_rg_1_2():
    rg = RowGroup(height=1, width=2)
    result = (
        "<table:table-row-group>"
        "<table:table-row><table:table-cell/><table:table-cell/>"
        "</table:table-row>"
        "</table:table-row-group>"
    )
    assert rg.serialize() == result


def test_create_rg_2_2():
    rg = RowGroup(height=2, width=2)
    result = (
        "<table:table-row-group>"
        "<table:table-row><table:table-cell/><table:table-cell/>"
        "</table:table-row>"
        "<table:table-row><table:table-cell/><table:table-cell/>"
        "</table:table-row>"
        "</table:table-row-group>"
    )
    assert rg.serialize() == result


def test_table_row_groups(table):
    assert len(table.row_groups) == 1


def test_table_row_groups_rows(table):
    rg = table.row_groups[0]
    count = 0
    for item in rg.children:
        if item.tag == "table:table-row":
            count += 1
    assert count == 2


def test_table_all_rows(table):
    assert len(table.rows) == 3


def test_table_cell_repeated_1(table):
    row1 = table.get_row(1)
    assert row1.get_values() == ["a2", "b2"]
    cell = table.get_cell("a2", clone=False)
    assert not cell.repeated
    cell_row = cell.parent
    rg = cell_row.parent
    assert isinstance(cell_row, Row)
    assert isinstance(rg, RowGroup)


def test_table_cell_repeated_2(table):
    cell = table.get_cell("a2", clone=False)
    cell.repeated = 3
    assert cell.repeated == 3
    row = cell.parent
    assert row.get_values() == ["a2", "a2", "a2", "b2"]



# Copyright 2018-2026 Jérôme Dumonteil
# Copyright (c) 2009-2010 Ars Aperta, Itaapy, Pierlis, Talend.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
#
# Authors (odfdo project): jerome.dumonteil@gmail.com
# The odfdo project is a derivative work of the lpod-python project:
# https://github.com/lpod/lpod-python

from __future__ import annotations

from collections.abc import Iterable

import pytest

from odfdo import Element
from odfdo.shapes import DrawPageThumbnail


@pytest.fixture
def page_thumbnail() -> Iterable[DrawPageThumbnail]:
    shape = DrawPageThumbnail(
        name="some name",
        page_number=42,
        position=("1cm", "2cm"),
        size=("10cm", "12cm"),
        placeholder=True,
        user_transformed=True,
    )
    yield shape


def test_draw_page_thumbnail_minimal():
    page_thumbnail = DrawPageThumbnail()
    assert (
        page_thumbnail._canonicalize() == "<draw:page-thumbnail></draw:page-thumbnail>"
    )


def test_draw_page_thumbnail_class():
    page_thumbnail = Element.from_tag("<draw:page-thumbnail/>")
    assert isinstance(page_thumbnail, DrawPageThumbnail)


def test_draw_page_thumbnail_position(page_thumbnail):
    assert page_thumbnail.position == ("1cm", "2cm")


def test_draw_page_thumbnail_size(page_thumbnail):
    assert page_thumbnail.size == ("10cm", "12cm")


def test_draw_page_thumbnail_page_number(page_thumbnail):
    assert page_thumbnail.page_number == 42


def test_draw_page_thumbnail_page_number_none(page_thumbnail):
    page_thumbnail.page_number = None
    assert page_thumbnail.page_number is None


def test_draw_page_thumbnail_placeholder(page_thumbnail):
    assert page_thumbnail.placeholder is True


def test_draw_page_thumbnail_user_transformed(page_thumbnail):
    assert page_thumbnail.user_transformed is True



#!/usr/bin/env python
"""Change the default language of a document, both in metadata and default styles."""

import os
import sys
from pathlib import Path

from odfdo import Document

_DOC_SEQUENCE = 11
DATA = Path(__file__).parent / "data"
SOURCE = "lorem.odt"
OUTPUT_DIR = Path(__file__).parent / "recipes_output" / "language_text"
TARGET = "lorem_en_us.odt"


def read_source_document() -> Document:
    """Return the source Document."""
    try:
        source = sys.argv[1]
    except IndexError:
        source = DATA / SOURCE
    return Document(source)


def save_new(document: Document, name: str) -> None:
    """Save a recipe result Document."""
    OUTPUT_DIR.mkdir(parents=True, exist_ok=True)
    new_path = OUTPUT_DIR / name
    print("Saving:", new_path)
    document.save(new_path, pretty=True)


def change_language(document: Document) -> None:
    current_language = document.language
    print(f"Current default language: {current_language!r}")

    document.language = "en-US"
    new_language = document.language
    print(f"New default language: {new_language!r}")


def main() -> None:
    document = read_source_document()
    change_language(document)
    test_unit(document)
    save_new(document, TARGET)


def test_unit(document: Document) -> None:
    # only for test suite:
    if "ODFDO_TESTING" not in os.environ:
        return

    assert document.language == "en-US"


if __name__ == "__main__":
    main()



#!/usr/bin/env python
"""Add not printable annotations to a document.

Annotations are notes that do not appear in the document but typically
on a side bar in a desktop application. So they are not printed.
"""

import os
from pathlib import Path

from odfdo import Document, Header, Paragraph

_DOC_SEQUENCE = 110
OUTPUT_DIR = Path(__file__).parent / "recipes_output" / "annotated"
TARGET = "annotated_document.odt"


def save_new(document: Document, name: str) -> None:
    """Save a recipe result Document."""
    OUTPUT_DIR.mkdir(parents=True, exist_ok=True)
    new_path = OUTPUT_DIR / name
    print("Saving:", new_path)
    document.save(new_path, pretty=True)


def base_document() -> Document:
    """Generate a basic document."""
    document = Document("text")
    body = document.body

    body.append(Header(1, "De la Guerre des Gaules - Livre V"))
    body.append(Header(2, "Préparatifs d'expédition en Bretagne"))
    body.append(
        Paragraph(
            "Sous le consulat de Lucius Domitius et d'Appius Claudius, "
            "César, quittant les quartiers d'hiver pour aller en Italie, "
            "comme il avait coutume de le faire chaque année, ordonne aux "
            "lieutenants qu'il laissait à la tête des légions de construire, "
            "pendant l'hiver, le plus de vaisseaux qu'il serait possible, "
            "et de réparer les anciens."
        )
    )
    body.append(Header(2, "La Bretagne"))
    body.append(
        Paragraph(
            "Cette île est de forme triangulaire ; l'un des côtés regarde "
            "la Gaule. Des deux angles de ce côté, l'un est au levant, "
            "vers le pays de Cantium, où abordent presque tous les vaisseaux "
            "gaulois ; l'autre, plus bas, est au midi. La longueur de ce côté "
            "est d'environ cinq cent mille pas. "
        )
    )
    return document


def insert_annotation(document: Document) -> None:
    """Insert a not printable annotation in a document."""
    body = document.body
    paragraph = body.get_paragraph(content="consulat")
    # Annotations are inserted like notes but they are simpler:
    # Annotation arguments:
    # after   =>  The word after what the annotation is inserted.
    # body    =>  The annotation itself, at the end of the page.
    # creator =>  The author of the annotation.
    # date    =>  A datetime value, by default datetime.now().
    paragraph.insert_annotation(
        after="Domitius",
        body="Talking about Lucius Domitius",
        creator="Luis",
    )


def main() -> None:
    document = base_document()
    insert_annotation(document)
    test_unit(document)
    save_new(document, TARGET)


def test_unit(document: Document) -> None:
    # only for test suite:
    if "ODFDO_TESTING" not in os.environ:
        return

    assert len(document.body.get_annotations(creator="Luis")) == 1


if __name__ == "__main__":
    main()



#!/usr/bin/env python
"""Adding a manual page break to a text document.

Page breaks are build by a specific style. However, odfdo provides a PageBreak
class to facilitate the inclusion of page breaks. This recipe illustrates
the use of PageBreak and the underlying styling mechanism.
"""

import os
from pathlib import Path

from odfdo import Document, PageBreak, Paragraph, Style

_DOC_SEQUENCE = 95
OUTPUT_DIR = Path(__file__).parent / "recipes_output" / "page_break"
TARGET = "document.odt"


def save_new(document: Document, name: str) -> None:
    """Save a recipe result Document."""
    OUTPUT_DIR.mkdir(parents=True, exist_ok=True)
    new_path = OUTPUT_DIR / name
    print("Saving:", new_path)
    document.save(new_path, pretty=True)


def generate_document() -> Document:
    """Return a text document containing page breaks."""
    document = Document()
    body = document.body
    body.clear()

    # here a simple way to insert a page break with odfdoshortcuts:
    document.add_page_break_style()
    body.append(Paragraph("First paragraph"))
    body.append(PageBreak())
    body.append(Paragraph("Second paragraph"))

    # here is a different way to insert a page break:
    page_break_style = Style("paragraph", name="page_break_before")
    page_break_style.set_properties({"fo:break-before": "page"})
    document.insert_style(page_break_style)
    empty_paragraph = Paragraph("", style="page_break_before")
    body.append(empty_paragraph)
    body.append(Paragraph("Third paragraph"))

    return document


def main() -> None:
    document = generate_document()
    test_unit(document)
    save_new(document, TARGET)


def test_unit(document: Document) -> None:
    # only for test suite:
    if "ODFDO_TESTING" not in os.environ:
        return

    content = document.body.serialize(pretty=True)
    assert "First paragraph" in content
    assert "Second paragraph" in content
    assert "Third paragraph" in content
    assert '<text:p text:style-name="odfdopagebreak"></text:p>' in content
    assert '<text:p text:style-name="page_break_before"></text:p>' in content


if __name__ == "__main__":
    main()



#!/usr/bin/env python
"""Put an image in a Cell of a table of a spreadsheet.

Possible sequence of operations:

    - put image content in the Document,
    - put the image internal URI in a Frame,
    - adapt the Frame parameters
    - put the Frame in the Table
"""

import os
from pathlib import Path

from odfdo import Cell, Document, Element, Frame, Table

_DOC_SEQUENCE = 58
OUTPUT_DIR = Path(__file__).parent / "recipes_output" / "image_table_sheet"
TARGET = "spreadsheet.ods"
DATA = Path(__file__).parent / "data"
LOGO = DATA / "newlogo.png"


def save_new(document: Document, name: str) -> None:
    """Save a recipe result Document."""
    OUTPUT_DIR.mkdir(parents=True, exist_ok=True)
    new_path = OUTPUT_DIR / name
    print("Saving:", new_path)
    document.save(new_path, pretty=True)


def add_row_style(document: Document, row_height: str) -> str:
    """Add some Row style to the document to adapt row height.

    Returns the style name"""
    style_def = f"""
    <style:style style:family="table-row">
        <style:table-row-properties style:row-height="{row_height}"
        fo:break-before="auto"/>
    </style:style>
    """
    style = Element.from_tag(style_def)
    style.name = f"row-height-{row_height}"
    # load the cell style in document
    document.insert_style(style, automatic=True)
    return style.name


def generate_document(image_path: Path, cell_coord: str) -> Document:
    document = Document("ods")
    body = document.body
    body.clear()

    table = Table("SomeTable")
    body.append(table)

    # insert the image content in the document
    uri = document.add_file(image_path)

    # image must be embedded in a Frame
    # Use the "char" anchor
    image_frame = Frame.image_frame(uri, anchor_type="char")

    # adapt the Frame for the table context:
    width, height = image_frame.size
    image_frame.set_attribute("table:end-x", width)
    image_frame.set_attribute("table:end-y", height)
    address = f"{table.name}.{cell_coord}"
    image_frame.set_attribute("table:end-cell-address", address)

    # create a new Cell to receive the image:
    cell = Cell()
    cell.append(image_frame)

    # insert the cell in the table:
    table.set_cell(cell_coord, cell)

    # optional: apply to the row the height of the image:
    style_name = add_row_style(document, height)
    # here we specifies "clone=False" to interact with the inplace row, not a copy:
    row = table.get_row(cell_coord, clone=False)
    row.style = style_name

    return document


def main() -> None:
    document = generate_document(LOGO, "B2")
    test_unit(document, "B2")
    save_new(document, TARGET)


def test_unit(document: Document, coord: str) -> None:
    # only for test suite:
    if "ODFDO_TESTING" not in os.environ:
        return

    table = document.body.get_table(0)
    cell = table.get_cell(coord)
    frame = cell.get_frames()[0]
    image = frame.images[0]
    url = image.url
    assert url.endswith("png")


if __name__ == "__main__":
    main()



#!/usr/bin/env python
"""Create an empty text document and add a picture in a frame,
aligned to the right or to the left.

Aligning an image requires applying a style to the frame. To do
this, use the default frame position style and customize it. The
frame position style allows you to choose alignment relative to
the paragraph (default) or the page.
"""

import os
from pathlib import Path

from odfdo import Document, Frame, Paragraph, default_frame_position_style

_DOC_SEQUENCE = 66
OUTPUT_DIR = Path(__file__).parent / "recipes_output" / "basic_picture_right"
TARGET = "document.odt"
DATA = Path(__file__).parent / "data"
IMAGE = DATA / "newlogo.png"


def save_new(document: Document, name: str) -> None:
    """Save a recipe result Document."""
    OUTPUT_DIR.mkdir(parents=True, exist_ok=True)
    new_path = OUTPUT_DIR / name
    print("Saving:", new_path)
    document.save(new_path, pretty=True)


def make_document() -> Document:
    """Generate a document containing two instances of an image,
    align one left, the other right.
    """
    document = Document("text")

    # add an image to the document, remember its URI
    image_path = str(DATA / IMAGE)
    uri = document.add_file(image_path)

    # add a frame style to the document, right alignment
    right_style = default_frame_position_style(
        name="right_frame", horizontal_pos="right"
    )
    document.insert_style(right_style)

    # add a frame style to the document, left alignment
    left_style = default_frame_position_style(
        name="left_frame", horizontal_pos="left", horizontal_rel="page"
    )
    document.insert_style(left_style)

    # make the image frames using previous informations
    image_frame_right = Frame.image_frame(
        uri,
        size=("6cm", "4cm"),
        position=("0cm", "5cm"),
        style=right_style.name,
    )
    image_frame_left = Frame.image_frame(
        uri,
        size=("9cm", "6cm"),
        position=("0cm", "12cm"),
        style=left_style.name,
    )

    # put image frame in a paragraph:
    paragraph = Paragraph("")
    paragraph.append(image_frame_right)
    paragraph.append(image_frame_left)
    document.body.append(paragraph)

    return document


def main() -> None:
    document = make_document()
    test_unit(document)
    save_new(document, TARGET)


def test_unit(document: Document) -> None:
    # only for test suite:
    if "ODFDO_TESTING" not in os.environ:
        return

    assert len([s for s in document.get_styles() if s.family == "graphic"]) >= 2
    graphic_style_names = [s.name for s in document.get_styles("graphic")]
    assert "right_frame" in graphic_style_names
    assert "left_frame" in graphic_style_names


if __name__ == "__main__":
    main()



# Copyright 2018-2026 Jérôme Dumonteil
# Copyright (c) 2009-2013 Ars Aperta, Itaapy, Pierlis, Talend.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
#
# Authors (odfdo project): jerome.dumonteil@gmail.com
# The odfdo project is a derivative work of the lpod-python project:
# https://github.com/lpod/lpod-python
"""Command-line script to export a table from an ODF spreadsheet to CSV.

This script extracts a specified table (or the first table by default) from
an ODS file and converts it to a CSV format, outputting to a file or
standard output.
"""

from __future__ import annotations

import sys
from argparse import ArgumentParser, Namespace

from odfdo import __version__
from odfdo.utils.script_utils import read_document

PROG = "odfdo-to-csv"


def configure_parser() -> ArgumentParser:
    description = (
        "Export a table from an ODS (OpenDocument Spreadsheet) file to a CSV file. "
        "The script extracts a specified table (or the first table by default) "
        "and converts its content into CSV format."
    )
    epilog = (
        "This tool provides a basic CSV exporter, leveraging the Python CSV module's "
        "functionality. It can output to a file or standard output. "
        "You can choose between 'excel' (default) or 'unix' CSV dialects."
    )
    parser = ArgumentParser(prog=PROG, description=description, epilog=epilog)
    parser.add_argument(
        "--version",
        action="version",
        version=f"{PROG} v{__version__}",
    )
    parser.add_argument(
        "-i",
        "--input",
        action="store",
        dest="input_file",
        metavar="INPUT",
        required=False,
        default=None,
        help="input .ods file if option not present, read from stdin",
    )
    parser.add_argument(
        "-o",
        "--output",
        action="store",
        dest="output_file",
        metavar="OUTPUT",
        required=False,
        help="output CSV file, if option not present, write to stdout",
    )

    parser.add_argument(
        "-t",
        "--table",
        action="store",
        dest="table_name",
        metavar="TABLE",
        required=False,
        help="table name, if option not present, default to the first table",
    )

    parser.add_argument(
        "-u",
        "--unix",
        action="store_true",
        default=False,
        help="use 'unix' dialect for CSV format, default is 'excel'",
    )
    return parser


def parse_cli_args(cli_args: list[str] | None = None) -> Namespace:
    parser = configure_parser()
    return parser.parse_args(cli_args)


def to_csv(args: Namespace) -> None:
    document = read_document(args.input_file)
    if document.get_type() not in {"spreadsheet", "spreadsheet-template"}:
        raise TypeError("Document must be a Spreadsheet type.")
    if args.table_name:
        table = document.body.get_table(name=args.table_name)
        if not table:
            raise ValueError(f"Table {args.table_name!r} not found")
    else:
        table = document.body.get_table()
        if not table:  # pragma: no cover
            raise ValueError("No table found")
    if args.unix:
        dialect = "unix"
    else:
        dialect = "excel"
    content = table.to_csv(path_or_file=args.output_file, dialect=dialect)
    if content is not None:  # pragma: no cover
        sys.stdout.buffer.write(content.encode())


def main() -> int:
    args: Namespace = parse_cli_args()
    return main_to_csv(args)


def main_to_csv(args: Namespace) -> int:
    try:
        to_csv(args)
    except Exception:
        configure_parser().print_help()
        print()
        raise
    return 0


if __name__ == "__main__":
    raise SystemExit(main())



# Copyright 2018-2026 Jérôme Dumonteil
# Copyright (c) 2009-2013 Ars Aperta, Itaapy, Pierlis, Talend.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
#
# Authors (odfdo project): jerome.dumonteil@gmail.com
# The odfdo project is a derivative work of the lpod-python project:
# https://github.com/lpod/lpod-python
"""Classes related to "style:master-page"."""

from __future__ import annotations

from typing import Any

from .element import Element, PropDef, PropDefBool, register_element_class
from .mixin_list import ListMixin
from .mixin_toc import TocMixin
from .office_forms import OfficeFormsMixin
from .section import SectionMixin
from .style_base import StyleBase
from .tracked_changes import TrackedChangesMixin
from .user_field_declaration import UserFieldDeclContMixin
from .variable_declaration import VarDeclMixin


class StyleMasterPage(OfficeFormsMixin, StyleBase):
    """Container of headers and footers, "style:master-page".

    In text and spreadsheet documents, the "style:master-page" element contains
    the content of headers and footers. For these types of documents, consumers
    may generate a sequence of pages by making use of a single master page or a
    set of master pages.

    In drawing and presentation documents, the "style:master-page" element is
    used to define master pages as common backgrounds for drawing pages. Each
    drawing page is directly linked to one master page, which is specified by
    the draw:master-page-name attribute of the drawing pages style.

    Master pages are contained in the "office:master-styles" element.

    All documents shall contain at least one master page element.

    If a text or spreadsheet document is displayed in a paged layout, master
    pages are used to generate a sequence of pages containing the document
    content. When a page is created, an empty page is generated with the
    properties of the master page and the static content of the master page.
    The body of the page is then filled with content. A single master pages can
    be used to created multiple pages within a document.

    In text and spreadsheet documents, a master page can be assigned to
    paragraph and table styles using a style:master-page-name attribute. Each
    time the paragraph or table style is applied to text, a page break is
    inserted before the paragraph or table. A page that starts at the page
    break position uses the specified master page.

    In drawings and presentations, master pages can be assigned to drawing
    pages using a style:parent-style-name attribute.

    The "style:master-page" element is usable within the following element:
    "office:master-styles".
    """

    # The <style:master-page> element has the following attributes:
    # draw:style-name
    # style:display-name
    # style:name
    # style:next-style-name
    # style:page-layout-name

    _tag: str = "style:master-page"
    _properties: tuple[PropDef | PropDefBool, ...] = (
        PropDef("name", "style:name"),
        PropDef("display_name", "style:display-name"),
        PropDef("page_layout", "style:page-layout-name", "master-page"),
        PropDef(
            "next_style",
            "style:next-style-name",
            "master-page",
        ),
        PropDef("draw_style_name", "draw:style-name"),
    )

    def __init__(
        self,
        name: str | None = None,
        display_name: str | None = None,
        page_layout: str | None = None,
        next_style: str | None = None,
        draw_style_name: str | None = None,
        **kwargs: Any,
    ) -> None:
        """Initialize a StyleMasterPage.

        The master page's `name` is not mandatory at creation but becomes
        required when inserted into a document as a common style.
        The `display_name` is the name visible to the user in office applications.

        Args:
            name: The internal name of the master page style.
            display_name: The user-facing display name of the style.
            page_layout: The name of the page layout style to use.
            next_style: The name of the next master page style to apply.
            draw_style_name: The name of the drawing style to apply.
            **kwargs: Additional keyword arguments for the parent `Element` class.
        """
        self._family = "master-page"
        tag_or_elem = kwargs.get("tag_or_elem")
        if tag_or_elem is None:
            kwargs["tag"] = "style:master-page"
        Element.__init__(self, **kwargs)
        if self._do_init:
            # print("_do_init")
            kwargs.pop("tag", None)
            kwargs.pop("tag_or_elem", None)
            if name:
                self.name = name
            if display_name:
                self.display_name = display_name
            if page_layout:
                self.page_layout = page_layout
            if next_style:
                self.next_style = next_style
            if draw_style_name:
                self.draw_style_name = draw_style_name

    @property
    def family(self) -> str | None:
        """Get the family of the style.

        For `StyleMasterPage`, this is always "master-page".
        """
        return self._family

    @family.setter
    def family(self, family: str | None) -> None:
        """Setter for the family property (no-op as family is fixed)."""
        pass

    def __repr__(self) -> str:
        return f"<{self.__class__.__name__} family={self.family} name={self.name}>"

    def _set_header_or_footer(
        self,
        text_or_element: str | Element | list[Element | str],
        name: str = "header",
        style: str = "Header",
    ) -> None:
        """Internal helper to set the content of a page header or footer.

        This method replaces existing content or creates a new header/footer
        element if one doesn't exist. It can accept raw text, an `Element`,
        or a list of `Element`s or strings.

        Args:
            text_or_element: The content to set. Can be a string, an `Element`,
                or an iterable of strings and/or `Element`s.
            name: The name of the part to set ("header" or "footer").
            style: The default style name to apply to new paragraphs
                created from string content.
        """
        if name == "header":
            header_or_footer = self.get_page_header()
        else:
            header_or_footer = self.get_page_footer()
        if header_or_footer is None:
            header_or_footer = Element.from_tag("style:" + name)
            self.append(header_or_footer)
        else:
            header_or_footer.clear()
        if (
            isinstance(text_or_element, Element)
            and text_or_element.tag == f"style:{name}"
        ):
            # Already a header or footer?
            self.delete(header_or_footer)
            self.append(text_or_element)
            return
        if isinstance(text_or_element, (Element, str)):
            elem_list: list[Element | str] = [text_or_element]
        else:
            elem_list = text_or_element
        for item in elem_list:
            if isinstance(item, str):
                paragraph = Element.from_tag("text:p")
                paragraph.append_plain_text(item)  # type: ignore
                paragraph.style = style  # type: ignore
                header_or_footer.append(paragraph)
            elif isinstance(item, Element):
                header_or_footer.append(item)

    def get_page_header(self) -> Element | None:
        """Get the `style:header` element containing the header contents.

        Returns:
            Element | None: The `style:header` element, or `None` if no header
                content is defined for this master page.
        """
        return self.get_element("style:header")

    def set_page_header(
        self,
        text_or_element: str | Element | list[Element | str],
    ) -> None:
        """Set or replace the content of the page header.

        This method uses `_set_header_or_footer` to update the header's content.
        You can provide raw text, an `Element`, or a list of content to replace
        the existing header. To modify the existing header without replacing it
        entirely, retrieve it first using `get_page_header` and use its API.

        Args:
            text_or_element: The new content for the header.
        """
        self._set_header_or_footer(text_or_element)

    def get_page_footer(self) -> Element | None:
        """Get the `style:footer` element containing the footer contents.

        Returns:
            Element | None: The `style:footer` element, or `None` if no footer
                content is defined for this master page.
        """
        return self.get_element("style:footer")

    def set_page_footer(
        self,
        text_or_element: str | Element | list[Element | str],
    ) -> None:
        """Set or replace the content of the page footer.

        This method uses `_set_header_or_footer` to update the footer's content.
        You can provide raw text, an `Element`, or a list of content to replace
        the existing footer. To modify the existing footer without replacing it
        entirely, retrieve it first using `get_page_footer` and use its API.

        Args:
            text_or_element: The new content for the footer.
        """
        self._set_header_or_footer(text_or_element, name="footer", style="Footer")

    def set_font(
        self,
        name: str,
        family: str | None = None,
        family_generic: str | None = None,
        pitch: str = "variable",
    ) -> None:
        """This method is not applicable to `StyleMasterPage` and does nothing."""


StyleMasterPage._define_attribut_property()


class StyleHeader(
    ListMixin,
    TocMixin,
    UserFieldDeclContMixin,
    VarDeclMixin,
    TrackedChangesMixin,
    SectionMixin,
    StyleBase,
):
    """Content of a header in a StyleMasterPage, tag "style:header".

    The "style:display" attribute specifies whether the header is displayed or
    not ("true" or "false"). The default value is "true".
    """

    _tag: str = "style:header"

    def __init__(
        self,
        display: str | bool | None = True,
        # Every other property
        **kwargs: Any,
    ) -> None:
        """Initialize a StyleHeader element.

        Args:
            display: Specifies whether the header is displayed.
                Can be "true", "false", or a boolean. Defaults to True.
            **kwargs: Additional keyword arguments for the parent `Element` class.
        """
        super().__init__(**kwargs)
        if self._do_init:
            kwargs.pop("tag", None)
            kwargs.pop("tag_or_elem", None)
            self.display = display

    @property
    def display(self) -> bool:
        """Get the display status of the header.

        Returns:
            bool: True if the header is displayed, False otherwise.
        """
        return self._get_attribute_bool_default("style:display", True)

    @display.setter
    def display(self, display: bool | str | None) -> None:
        """Set the display status of the header.

        Args:
            display: The display status. Can be a boolean,
                or "true"/"false" string.
        """
        self._set_attribute_bool_default("style:display", display, True)

    def __repr__(self) -> str:
        return f"<{self.__class__.__name__}>"


class StyleFooter(StyleHeader):
    """Content of a footer in a StyleMasterPage, tag "style:footer".

    The "style:display" attribute specifies whether the footer is displayed or
    not ("true" or "false"). The default value is "true".
    """

    _tag: str = "style:footer"


class StyleHeaderLeft(StyleHeader):
    """Content of a left page header in a StyleMasterPage, tag "style:header-
    left".

    If left page different from the right page in a "style:master-page".

    The "style:display" attribute specifies whether the header is displayed or
    not ("true" or "false"). The default value is "true".
    """

    _tag: str = "style:header-left"


class StyleHeaderFirst(StyleHeader):
    """Content of a header for a first page, if different from the left/right
    page in a "style:master-page" element, tag "style:header-first"

    The term "first page" means the first page to which the page style is
    applied, regardless of any numbering. If a different page style is
    applied in between two applications of a page style that has the
    "style:header-first" element, each of the applications of the page
    style has its own first page.

    The "style:display" attribute specifies whether the header is displayed or
    not ("true" or "false"). The default value is "true".

    New in ODFv1.3.
    """

    _tag: str = "style:header-first"


class StyleFooterLeft(StyleHeader):
    """Content of a left page footer in a StyleMasterPage, tag "style:footer-
    left".

    If left page different from the right page in a "style:master-page".

    The "style:display" attribute specifies whether the footer is displayed or
    not ("true" or "false"). The default value is "true".
    """

    _tag: str = "style:footer-left"


class StyleFooterFirst(StyleHeader):
    """Content of a footer for a first page, if different from the left/right
    page in a "style:master-page" element, tag "style:footer-first".

    The term "first page" means the first page to which the page style is
    applied, regardless of any numbering. If a different page style is
    applied in between two applications of the page style, that has the
    "style:footer-first" element, each of the applications of the page style
    has its own first page.

    New in ODFv1.3.
    """

    _tag: str = "style:footer-first"


StyleFooterLeft._define_attribut_property()

register_element_class(StyleFooter)
register_element_class(StyleFooterFirst)
register_element_class(StyleFooterLeft)
register_element_class(StyleHeader)
register_element_class(StyleHeaderFirst)
register_element_class(StyleHeaderLeft)
register_element_class(StyleMasterPage)



# Copyright 2018-2026 Jérôme Dumonteil
# Copyright (c) 2009-2010 Ars Aperta, Itaapy, Pierlis, Talend.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
#
# Authors (odfdo project): jerome.dumonteil@gmail.com
# The odfdo project is a derivative work of the lpod-python project:
# https://github.com/lpod/lpod-python
import io
from collections.abc import Iterable

import pytest

from odfdo import Document, Frame, Paragraph


@pytest.fixture
def doc_xml(samples) -> Iterable:
    doc = Document("odt")
    doc.body.clear()
    img_url = doc.add_file(samples("image.png"))
    image_frame = Frame.image_frame(
        img_url,
        size=("6cm", "4cm"),
        position=("5cm", "10cm"),
    )
    paragraph = Paragraph()
    paragraph.append(image_frame)
    doc.body.append(paragraph)
    with io.BytesIO() as bytes_content:
        doc.save(bytes_content, packaging="xml")
        xml_content = bytes_content.getvalue()
    yield xml_content.decode()


def test_save_xml_header(doc_xml):
    assert doc_xml.startswith('<?xml version="1.0" encoding="UTF-8"?>')


def test_save_xml_header_2(doc_xml):
    assert (
        'office:version="1.2" office:mimetype="application/vnd.oasis.opendocument.text">'
        in doc_xml
    )


def test_save_xml_header_meta(doc_xml):
    assert "<office:meta>" in doc_xml
    assert "    <meta:creation-date>" in doc_xml


def test_save_xml_settings(doc_xml):
    assert "<office:settings>" in doc_xml
    assert "<office:font-face-decls>" in doc_xml


def test_save_xml_styles(doc_xml):
    assert "  <office:styles>" in doc_xml
    assert "<office:font-face-decls>" in doc_xml
    assert "  </office:master-styles>" in doc_xml
    assert "<office:scripts/>" in doc_xml


def test_save_xml_content_1(doc_xml):
    assert "  <office:body>" in doc_xml
    assert "    <office:text>" in doc_xml


def test_save_xml_content_2(doc_xml):
    assert "  <office:body>" in doc_xml
    assert (
        '<text:p><draw:frame svg:width="6cm" svg:height="4cm" draw:z-index="0" '
        'svg:x="5cm" svg:y="10cm" text:anchor-type="paragraph">'
    ) in doc_xml


def test_save_xml_content_3(doc_xml):
    assert (
        """<office:body>\n    <office:text>\n      <text:p><draw:frame svg:width="6cm" svg:height="4cm" draw:z-index="0" svg:x="5cm" svg:y="10cm" text:anchor-type="paragraph">\n          <draw:image>\n            <office:binary-data>iVBORw0KGgoAAAANSUhEUgAAAQEAAACWCAIAAABRmqrlAAA\n             gAElEQVR42sy9WbNl13Emlrn23me881BzFVAAAQICAYiSKFJqWU1Ft4ZWh93q6AjZc\n             jjkB3dH2yF32B1hh/vF0WH7wREOv/lFP6LpoEJPDskaqHATpCgQ4NAFEFUACjXeeTj\n             nnmEPmX5Ya2Xm2qcIVBVAUZeMwq17b91z9t5rrcz88vu+xBf+xXMAgIwAzAgAAAwAA\n             AjIwIgIzAwQPkFEZv8D/qcQgIEBkAH8nwQA6IjDX+XrjMiAxEDgCJDANQjMiOgQ0QE\n             6xPAXRAfh88VPEJ0DQEQHLnwLAcE5ROeyHF2OWeZc7lyGLndZji5zLnNZ7tABEwEzE\n             zMQEDExEwMTM5P/FhMQ+y8AMYP/GQImYgAK3/P/FpiIgBqmBpqGqQEmoAaoZiKgBpo\n             GuGEiaOJXqAFuuGkg/pWpAarlr0ANE4FzDh26zKFD5xAzlyGg/ys6f9X+v+ET5wbzH\n             AGA/bNADI8RgPzTC8/I+S8TICAAuBxdjlg4V6ArHBbgCuc6iDm6wmUdhwW6wrkOYOF\n             c7rIOxq84V6DrOHQAfvUQMDETQgPcINcMDXINXCPXTBVyDVwBl8g1UIl665kYiInYP\n             wD2n5C/0ez/JPvDzEwEFJ4kNEThJ4mponnZVHVTVvW8pLJuyqopAQEY/J8Y7gtwuCH\n             +1iCzX/1hHwCwv2PM7O8VAyMgyO1Ffw/9BgD0X407IWwn9j/FGF7c/2v/Hvwbkt+F8\n             szis4q/DhEQ/Y+A7lH5xfKkARAZEBH92wmf+u3rHGLc5uxfhRniZuZ4f+J7jHvXX17\n             c9AjE/ivEHL/jP+N4IPhfDPItAGZk8lcWXpb8QvVvwX8Rw7+INzxcQrgN4XaZ2x6/7\n """
        in doc_xml
    )


def test_save_xml_content_4(doc_xml):
    assert (
        """hkbURB7PoCAhFR0bz/EXZ7JPVMSkBEgml5zuMUwo1moOKorWguyVhzeISVkKDbCxGN\n             UzWNqnRGbIfMI8bzeWyjgUs47CfqaNil6wJJrkPIBFi/SiwnhpJEuFmx2vNjm6BSoO\n             jw2AcUXtrC57rMOFUA1nEsXwy0DS836Bex0zmEW4c2MxYWmnZlk7TbO8psQCkFYqbm\n             bfGRioTF7e1k0LzdTRyXRrpPQ/aapClaKORetzz3eOB4BNm4iKyWaw5ihB4xIcAr4F\n             RbDmJyJm+Spui8eRW2TUR3uB2qGIbcneOLjyjZMNgvQnKAH/t6cmjLN+Bctd1Xy2mu\n             wFrh2O8FEukiITFzIiwFlq9c51gMoWIifw3Ge2aEHd+GkwAAAAASUVORK5CYII=\n            </office:binary-data>\n          </draw:image>\n        </draw:frame>\n      </text:p>\n    </office:text>\n  </office:body>\n</office:document>\n"""
        in doc_xml
    )



# Copyright 2018-2026 Jérôme Dumonteil
# Copyright (c) 2009-2010 Ars Aperta, Itaapy, Pierlis, Talend.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
#
# Authors (odfdo project): jerome.dumonteil@gmail.com
# The odfdo project is a derivative work of the lpod-python project:
# https://github.com/lpod/lpod-python
# Authors: Romain Gauthier <romain@itaapy.com>
#          Hervé Cauwelier <herve@itaapy.com>
#          David Versmisse <david.versmisse@itaapy.com>
#          Jerome Dumonteil <jerome.dumonteil@itaapy.com>

from collections.abc import Iterable

import pytest

from odfdo import Element
from odfdo.document import Document
from odfdo.table import Row, Table


@pytest.fixture
def body(samples) -> Iterable[Element]:
    document = Document(samples("simple_table.ods"))
    yield document.body


@pytest.fixture
def table(samples) -> Iterable[Table]:
    # simpletable :
    #   1	1	1	2	3	3	3
    #   1	1	1	2	3	3	3
    #   1	1	1	2	3	3	3
    #   1   2	3	4	5	6	7
    document = Document(samples("simple_table.ods"))
    yield document.body.get_table(name="Example1")


def test_get_table_list(body):
    assert len(body.get_tables()) == 3


def test_get_sheets_alias(body):
    assert body.get_tables == body.get_sheets


def test_get_sheets_list(body):
    assert len(body.get_sheets()) == 3


def test_get_table_list_property(body):
    assert len(body.tables) == 3


def test_get_sheets_list_property(body):
    assert len(body.sheets) == 3


def test_get_table_list_style(body):
    assert len(body.get_tables(style="ta1")) == 3


def test_get_table_by_name(body):
    name = "New Table"
    body.append(Table(name))
    table = body.get_table(name=name)
    assert table.name == name


def test_get_sheet_alias(body):
    assert body.get_sheet == body.get_table


def test_get_sheet_by_name(body):
    name = "New Table"
    body.append(Table(name))
    table = body.get_sheet(name=name)
    assert table.name == name


def test_get_table_by_position(body):
    body.append(Table("New Table"))
    table = body.get_table(position=3)
    assert table.name == "New Table"


def test_get_table_style(table):
    assert table.style == "ta1"


def test_get_table_printable(table):
    assert table.printable is False


def test_get_table_width(table):
    assert table.width == 7


def test_get_table_height(table):
    assert table.height == 4


def test_get_table_size(table):
    assert table.size == (7, 4)


def test_get_table_size_empty():
    table = Table("Empty")
    assert table.size == (0, 0)


def test_get_table_width_after():
    table = Table("Empty")
    assert table.width == 0
    assert table.height == 0
    # The first row creates the columns
    table.append_row(Row(width=5))
    assert table.width == 5
    assert table.height == 1
    # The subsequent ones don't
    table.append_row(Row(width=5))
    assert table.width == 5
    assert table.height == 2


def test_get_values(table):
    expected = [
        [1, 1, 1, 2, 3, 3, 3],
        [1, 1, 1, 2, 3, 3, 3],
        [1, 1, 1, 2, 3, 3, 3],
        [1, 2, 3, 4, 5, 6, 7],
    ]
    assert table.get_values() == expected


def test_set_table_values_with_clear(table):
    values = [
        ["a", "b", "c", "d", "e", "f", "g"],
        ["h", "i", "j", "k", "l", "m", "n"],
        ["o", "p", "q", "r", "s", "t", "u"],
        ["v", "w", "x", "y", "z", "aa", "ab"],
    ]
    table.clear()
    table.set_values(values)
    assert table.get_values() == values


def test_set_table_values_big(table):
    values = [
        ["a", "b", "c", "d", "e", "f", "g"],
        ["h", "i", "j", "k", "l", "m", "n"],
        ["o", "p", "q", "r", "s", "t", "u"],
        ["o", "p", "q", "r", "s", "t", "u"],
        ["o", "p", "q", "r", "s", "t", "u"],
        ["o", "p", "q", "r", "s", "t", "u"],
        ["v", "w", "x", "y", "z", "aa", "ab"],
        ["v", "w", "x", "y", "z", "aa", "ab"],
    ]
    table.set_values(values)
    assert table.get_values() == values
    assert table.size == (7, 8)


def test_set_table_values_small(table):
    values = [
        ["a", "b", "c"],
        ["h", "i", "j", "k", "l", "m", "n"],
        ["o", "p", None, None, "s", "t", "u"],
    ]
    table.set_values(values)
    assert table.size == (7, 4)
    assert table.get_values() == [
        ["a", "b", "c", 2, 3, 3, 3],
        ["h", "i", "j", "k", "l", "m", "n"],
        ["o", "p", None, None, "s", "t", "u"],
        [1, 2, 3, 4, 5, 6, 7],
    ]


def test_set_table_values_small_coord(table):
    values = [
        ["a", "b", "c"],
        ["h", "i", "j", "k", "l", "m", "n"],
        ["o", "p", None, None, "s", "t", "u"],
    ]
    table.set_values(values, coord=("c2"))
    assert table.size == (9, 4)
    assert table.get_values() == [
        [1, 1, 1, 2, 3, 3, 3, None, None],
        [1, 1, "a", "b", "c", 3, 3, None, None],
        [1, 1, "h", "i", "j", "k", "l", "m", "n"],
        [1, 2, "o", "p", None, None, "s", "t", "u"],
    ]


def test_set_table_values_small_coord_far(table):
    values = [["a", "b", "c"], ["h", None], ["o"]]
    table.set_values(values, coord=("J6"))
    assert table.size == (12, 8)
    expected = [
        [1, 1, 1, 2, 3, 3, 3, None, None, None, None, None],
        [1, 1, 1, 2, 3, 3, 3, None, None, None, None, None],
        [1, 1, 1, 2, 3, 3, 3, None, None, None, None, None],
        [1, 2, 3, 4, 5, 6, 7, None, None, None, None, None],
        [
            None,
            None,
            None,
            None,
            None,
            None,
            None,
            None,
            None,
            None,
            None,
            None,
        ],
        [None, None, None, None, None, None, None, None, None, "a", "b", "c"],
        [None, None, None, None, None, None, None, None, None, "h", None, None],
        [None, None, None, None, None, None, None, None, None, "o", None, None],
    ]
    assert table.get_values() == expected


def test_set_table_values_small_type(table):
    values = [[10, None, 30], [None, 40]]
    table.set_values(values, coord=("C4"), cell_type="percentage")
    assert table.size == (7, 5)
    assert table.get_values() == [
        [1, 1, 1, 2, 3, 3, 3],
        [1, 1, 1, 2, 3, 3, 3],
        [1, 1, 1, 2, 3, 3, 3],
        [1, 2, 10, None, 30, 6, 7],
        [None, None, None, 40, None, None, None],
    ]
    assert table.get_values(coord="4:", get_type=True) == [
        [
            (1, "float"),
            (2, "float"),
            (10, "percentage"),
            (None, None),
            (30, "percentage"),
            (6, "float"),
            (7, "float"),
        ],
        [
            (None, None),
            (None, None),
            (None, None),
            (40, "percentage"),
            (None, None),
            (None, None),
            (None, None),
        ],
    ]



# Copyright 2018-2026 Jérôme Dumonteil
# Copyright (c) 2009-2010 Ars Aperta, Itaapy, Pierlis, Talend.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
#
# Authors (odfdo project): jerome.dumonteil@gmail.com
# The odfdo project is a derivative work of the lpod-python project:
# https://github.com/lpod/lpod-python

from __future__ import annotations

from collections.abc import Iterable
from decimal import Decimal

import pytest

from odfdo import Element
from odfdo.form_properties import FormProperty


@pytest.fixture
def form_property() -> Iterable[FormProperty]:
    yield FormProperty(
        property_name="some name",
        boolean_value=True,
        currency="EUR",
        date_value="2021-01-01",
        string_value="some string",
        time_value="PT10H00M00S",
        value=Decimal("12.34"),
        value_type="float",
    )


def test_form_property_class():
    form = FormProperty()
    assert isinstance(form, FormProperty)


def test_form_property_minimal_tag():
    form = Element.from_tag("<form:property/>")
    assert isinstance(form, FormProperty)


def test_form_property_serialize():
    form = FormProperty()
    assert form.serialize() == "<form:property/>"


def test_form_property_property_name(form_property):
    assert form_property.property_name == "some name"


def test_form_property_boolean_value(form_property):
    assert form_property.boolean_value is True


def test_form_property_currency(form_property):
    assert form_property.currency == "EUR"


def test_form_property_date_value(form_property):
    assert form_property.date_value == "2021-01-01"


def test_form_property_string_value(form_property):
    assert form_property.string_value == "some string"


def test_form_property_time_value(form_property):
    assert form_property.time_value == "PT10H00M00S"


def test_form_property_value(form_property):
    assert form_property.value == Decimal("12.34")


def test_form_property_value_type(form_property):
    assert form_property.value_type == "float"



# Copyright 2018-2026 Jérôme Dumonteil
# Copyright (c) 2009-2010 Ars Aperta, Itaapy, Pierlis, Talend.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
#
# Authors (odfdo project): jerome.dumonteil@gmail.com
# The odfdo project is a derivative work of the lpod-python project:
# https://github.com/lpod/lpod-python
from __future__ import annotations

from collections.abc import Iterable

import pytest

from odfdo import Element
from odfdo.document import Document
from odfdo.draw_page import DrawPage
from odfdo.paragraph import Paragraph
from odfdo.shapes import ConnectorShape, EllipseShape, RectangleShape


@pytest.fixture
def body(samples) -> Iterable[Element]:
    document = Document(samples("base_shapes.odg"))
    yield document.body


@pytest.fixture
def connector() -> Iterable[ConnectorShape]:
    shape1 = RectangleShape(draw_id="31")
    shape2 = RectangleShape(draw_id="32")
    shape = ConnectorShape(
        name="some name",
        style="style name",
        text_style="text style",
        draw_id="draw_id",
        layer="layer",
        connected_shapes=(shape1, shape2),
        glue_points=("100", "200"),
        p1=("1cm", "2cm"),
        p2=("3cm", "4cm"),
        line_skew="5",
        svg_d="6",
        view_box="view box",
        presentation_class="pres class",
        presentation_style="pres style",
        caption_id="capid",
        class_names="class1",
        transform="tr1",
        z_index=4,
        end_cell_address="x10",
        end_x="10",
        end_y="20",
        table_background=True,
        anchor_type="page",
        anchor_page=42,
        xml_id="xmlid",
    )
    shape.svg_title = "title"
    yield shape


def test_connectorshape_minimal():
    shape = ConnectorShape()
    assert shape._canonicalize() == "<draw:connector></draw:connector>"


def test_connectorshape_class():
    shape = Element.from_tag("<draw:connector/>")
    assert isinstance(shape, ConnectorShape)


def test_connectorshape_connected_shapes_1(connector):
    assert connector.connected_shapes == ("31", "32")


def test_connectorshape_connected_shapes_2(connector):
    assert connector.start_shape == "31"


def test_connectorshape_connected_shapes_3(connector):
    assert connector.end_shape == "32"


def test_connectorshape_connected_shapes_4(connector):
    connector.connected_shapes = None
    assert connector.connected_shapes == (None, None)


def test_connectorshape_glue_points_1(connector):
    assert connector.glue_points == ("100", "200")


def test_connectorshape_glue_points_2(connector):
    assert connector.start_glue_point == "100"


def test_connectorshape_glue_points_3(connector):
    assert connector.end_glue_point == "200"


def test_connectorshape_glue_points_4(connector):
    connector.glue_points = None
    assert connector.glue_points == (None, None)


def test_connectorshape_p1_1(connector):
    assert connector.p1 == ("1cm", "2cm")


def test_connectorshape_p1_2(connector):
    assert connector.x1 == "1cm"


def test_connectorshape_p1_3(connector):
    assert connector.y1 == "2cm"


def test_connectorshape_p1_4(connector):
    connector.p1 = None
    assert connector.p1 == (None, None)


def test_connectorshape_p2_1(connector):
    assert connector.p2 == ("3cm", "4cm")


def test_connectorshape_p2_2(connector):
    assert connector.x2 == "3cm"


def test_connectorshape_p2_3(connector):
    assert connector.y2 == "4cm"


def test_connectorshape_p2_4(connector):
    connector.p2 = None
    assert connector.p2 == (None, None)


def test_connectorshape_draw_type_1(connector):
    assert connector.draw_type == "standard"


def test_connectorshape_draw_type_2(connector):
    connector.draw_type = "curve"
    assert connector.draw_type == "curve"


def test_connectorshape_draw_type_3(connector):
    with pytest.raises(TypeError):
        connector.draw_type = "bad"


def test_connectorshape_draw_type_4():
    connector = ConnectorShape(draw_type="line")
    assert connector.draw_type == "line"


def test_create_connectorshape():
    page = DrawPage("Page1")
    rectangle = RectangleShape(
        size=("10cm", "20cm"), position=("3cm", "4cm"), draw_id="rectangle"
    )
    ellipse = EllipseShape(
        size=("31cm", "32cm"), position=("35cm", "45cm"), draw_id="ellipse"
    )
    connector = ConnectorShape(
        connected_shapes=(rectangle, ellipse), glue_points=(1, 2)
    )
    page.append(rectangle)
    page.append(ellipse)
    page.append(connector)
    expected = (
        '<draw:page draw:id="Page1">'
        '<draw:rect draw:id="rectangle" '
        'svg:height="20cm" '
        'svg:width="10cm" '
        'svg:x="3cm" '
        'svg:y="4cm">'
        "</draw:rect>"
        '<draw:ellipse draw:id="ellipse" '
        'svg:height="32cm" '
        'svg:width="31cm" '
        'svg:x="35cm" '
        'svg:y="45cm">'
        "</draw:ellipse>"
        "<draw:connector "
        'draw:end-glue-point="2" '
        'draw:end-shape="ellipse" '
        'draw:start-glue-point="1" '
        'draw:start-shape="rectangle">'
        "</draw:connector>"
        "</draw:page>"
    )
    assert page._canonicalize() == expected


def test_page_get_draw_connector_list(body):
    page = body.get_draw_page()
    connectors = page.get_draw_connectors()
    assert len(connectors) == 1


def test_page_get_draw_connector_list_regex(body):
    page = body.get_draw_page()
    connectors = page.get_draw_connectors(content="Con")
    assert len(connectors) == 1


def test_page_get_draw_connector_list_draw_style(body):
    page = body.get_draw_page()
    connectors = page.get_draw_connectors(draw_style="gr4")
    assert len(connectors) == 0


def test_page_get_draw_connector_list_draw_text_style(body):
    page = body.get_draw_page()
    connectors = page.get_draw_connectors(draw_text_style="P1")
    assert len(connectors) == 1


def test_page_get_draw_connector_by_content(body):
    page = body.get_draw_page()
    connector = page.get_draw_connector(content="ecteur")
    expected = (
        '<draw:connector draw:end-glue-point="2" '
        'draw:end-shape="id2" '
        'draw:layer="layout" '
        'draw:start-glue-point="1" '
        'draw:start-shape="id1" '
        'draw:style-name="gr1" '
        'draw:text-style-name="P1" '
        'svg:d="M11000 8000h1250v1001h3250v-501" '
        'svg:viewBox="0 0 4501 1002" '
        'svg:x1="11cm" '
        'svg:x2="15.5cm" '
        'svg:y1="8cm" '
        'svg:y2="8.5cm">'
        '<text:p text:style-name="P1">'
        "Connecteur"
        "</text:p>"
        "</draw:connector>"
    )
    assert connector._canonicalize() == expected


def test_page_get_draw_connector_by_id(body):
    page = body.get_draw_page()
    connector = ConnectorShape(draw_id="an id")
    page.append(connector)
    connector = page.get_draw_connector(id="an id")
    expected = '<draw:connector draw:id="an id"></draw:connector>'
    assert connector._canonicalize() == expected


def test_page_get_draw_orphans_connector(body):
    # page = body.get_draw_page()
    orphan_connector = ConnectorShape()
    orphan_connector.append(Paragraph("Orphan c"))
    body.append(orphan_connector)
    connectors = body.get_orphan_draw_connectors()
    assert len(connectors) == 1



#!/usr/bin/env python
"""Create a basic document containing some paragraphs and headers, add some
footnotes. Footnotes are displayed at the end of the pages of the document.
"""

import os
from pathlib import Path

from odfdo import Document, Header, Paragraph

_DOC_SEQUENCE = 45
DATA = Path(__file__).parent / "data"
LOREM = (DATA / "lorem.txt").read_text(encoding="utf8")
OUTPUT_DIR = Path(__file__).parent / "recipes_output" / "basic_footnotes"
TARGET = "document.odt"


def save_new(document: Document, name: str) -> None:
    """Save a recipe result Document."""
    OUTPUT_DIR.mkdir(parents=True, exist_ok=True)
    new_path = OUTPUT_DIR / name
    print("Saving:", new_path)
    document.save(new_path, pretty=True)


def make_document() -> Document:
    """Generate a basic document containing some paragraphs and headers."""
    document = Document("text")
    body = document.body
    body.clear()

    level_1_title = Header(1, "Main title")
    body.append(level_1_title)

    for idx in range(3):
        level_2_title = Header(2, f"title {idx}")
        body.append(level_2_title)
        paragraph = Paragraph(LOREM[:240])
        body.append(paragraph)

    return document


def make_footnotes(document: Document) -> None:
    """Add some footnote for each pragraph of the document."""
    word_position = 0
    note_counter = 0
    for paragraph in document.body.paragraphs:
        # choosing some word of the paragraph to insert the note
        word_position += 3
        some_word = str(paragraph).split()[word_position]

        # Notes are quite complex so they deserve a dedicated API on paragraphs:
        note_counter += 10
        paragraph.insert_note(
            # The word after what the “¹” citation is inserted:
            after=some_word,
            # A unique identifier of the note in the document:
            note_id=f"note{note_counter}",
            # The symbol the user sees to follow the footnote:
            citation="1",
            # The footnote content itself:
            body=('John Doe, A. (2007). "How to cite references", Sample Editions.'),
        )


def main() -> None:
    document = make_document()
    make_footnotes(document)
    test_unit(document)
    save_new(document, TARGET)


def test_unit(document: Document) -> None:
    # only for test suite:
    if "ODFDO_TESTING" not in os.environ:
        return

    assert len(document.body.get_notes()) == 3


if __name__ == "__main__":
    main()



# Copyright 2018-2026 Jérôme Dumonteil
# Copyright (c) 2009-2010 Ars Aperta, Itaapy, Pierlis, Talend.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
#
# Authors (odfdo project): jerome.dumonteil@gmail.com
# The odfdo project is a derivative work of the lpod-python project:
# https://github.com/lpod/lpod-python
# Authors: Hervé Cauwelier <herve@itaapy.com>
#          David Versmisse <david.versmisse@itaapy.com>

from collections.abc import Iterable

import pytest
from lxml.etree import _ElementTree

from odfdo.const import ODF_CONTENT, ODF_META
from odfdo.container import Container
from odfdo.content import Content
from odfdo.element import Element
from odfdo.xmlpart import XmlPart


@pytest.fixture
def exemple_container(samples) -> Iterable[Container]:
    container = Container()
    container.open(samples("example.odt"))
    yield container


def test_get_element_list(exemple_container):
    content_part = XmlPart(ODF_CONTENT, exemple_container)
    elements = content_part.get_elements("//text:p")
    # The annotation paragraph is counted
    assert len(elements) == 8


def test_tree(exemple_container):
    # Testing a private but important method
    content = XmlPart(ODF_CONTENT, exemple_container)
    tree = content._get_tree()
    assert isinstance(tree, _ElementTree)
    assert content._XmlPart__tree is not None


def test_root(exemple_container):
    content = XmlPart(ODF_CONTENT, exemple_container)
    root = content.root
    assert isinstance(root, Element)
    assert root.tag == "office:document-content"
    assert content._XmlPart__root is not None


def test_serialize(exemple_container):
    content_bytes = exemple_container.get_part(ODF_CONTENT)
    content_part = XmlPart(ODF_CONTENT, exemple_container)
    # differences with lxml
    serialized = content_part.serialize().replace(b"'", b"&apos;")
    print(serialized)
    print(content_bytes)
    assert content_bytes == serialized


def test_pretty_serialize_base():
    # With pretty = True
    element = Element.from_tag("<root><a>spam</a><b/></root>")
    serialized = element.serialize(pretty=True)
    expected = "<root>\n  <a>spam</a>\n  <b/>\n</root>\n"
    assert serialized == expected


def test_clone(exemple_container):
    # Testing that the clone works on subclasses too
    content = Content(ODF_CONTENT, exemple_container)
    clone = content.clone
    assert clone.part_name == content.part_name
    assert id(exemple_container) != id(clone.container)
    assert clone._XmlPart__tree is None


def test_delete(exemple_container):
    content = XmlPart(ODF_CONTENT, exemple_container)
    paragraphs = content.get_elements("//text:p")
    for paragraph in paragraphs:
        content.delete_element(paragraph)
    serialized = content.serialize()
    assert serialized.count(b"<text:p") == 0


def test_repr(exemple_container):
    content_part = XmlPart(ODF_CONTENT, exemple_container)
    result = repr(content_part)
    assert result == "<XmlPart part_name=content.xml>"


def test_str(exemple_container):
    content_part = XmlPart(ODF_CONTENT, exemple_container)
    assert str(content_part) == repr(content_part)


def test_get_body(exemple_container):
    content_part = XmlPart(ODF_CONTENT, exemple_container)
    body = content_part.body
    assert body.tag == "office:text"


def test_get_body_none(exemple_container):
    content_part = XmlPart(ODF_META, exemple_container)
    with pytest.raises(TypeError):
        _ = content_part.body


def test_set_body(exemple_container):
    content_part = XmlPart(ODF_CONTENT, exemple_container)
    body = content_part.body
    body_clone = body.clone
    content_part.body = body_clone
    assert content_part.body.tag == "office:text"


def test_pretty_serialize_internal_not_pretty(samples):
    container = Container()
    container.open(samples("issue_28_pretty.odt"))
    content = XmlPart(ODF_CONTENT, container)
    serialized = content.serialize()
    expected = b'This is an example with =&gt;</text:span><text:span text:style-name="T4"> v</text:span><text:span text:style-name="T5">8</text:span><text:span text:style-name="T4">.1.</text:span><text:span text:style-name="T5">4</text:span><text:span text:style-name="T4"> &lt;</text:span><text:span text:style-name="T6">= spaces </text:span><text:span text:style-name="T7">after reading and writing with odfdo.'
    assert expected in serialized


def test_pretty_serialize_internal_pretty(samples):
    container = Container()
    container.open(samples("issue_28_pretty.odt"))
    content = XmlPart(ODF_CONTENT, container)
    serialized = content.pretty_serialize()
    print(serialized.decode())
    expected = b'This is an example with =&gt;</text:span><text:span text:style-name="T4"> v</text:span><text:span text:style-name="T5">8</text:span><text:span text:style-name="T4">.1.</text:span><text:span text:style-name="T5">4</text:span><text:span text:style-name="T4"> &lt;</text:span><text:span text:style-name="T6">= spaces </text:span><text:span text:style-name="T7">after reading and writing with odfdo.'
    assert expected in serialized


def test_pretty_serialize_internal_pretty2(samples):
    container = Container()
    container.open(samples("issue_28_pretty.odt"))
    content = XmlPart(ODF_CONTENT, container)
    serialized = content.serialize(pretty=True)
    expected = b'This is an example with =&gt;</text:span><text:span text:style-name="T4"> v</text:span><text:span text:style-name="T5">8</text:span><text:span text:style-name="T4">.1.</text:span><text:span text:style-name="T5">4</text:span><text:span text:style-name="T4"> &lt;</text:span><text:span text:style-name="T6">= spaces </text:span><text:span text:style-name="T7">after reading and writing with odfdo.'
    assert expected in serialized



# Copyright 2018-2026 Jérôme Dumonteil
# Authors (odfdo project): jerome.dumonteil@gmail.com
from __future__ import annotations

import subprocess
import sys
from pathlib import Path

import pytest

from odfdo.scripts import styles
from odfdo.scripts.styles import main as main_script
from odfdo.scripts.styles import main_styles, parse_cli_args

SCRIPT = Path(styles.__file__)


def run_params_bytes(params):
    command = [sys.executable, SCRIPT] + params
    proc = subprocess.Popen(
        command,
        stdout=subprocess.PIPE,
        stderr=subprocess.PIPE,
    )
    out, err = proc.communicate()
    return out, err, proc.returncode


def test_styles_no_param():
    params = []
    out, err, exitcode = run_params_bytes(params)
    print(out, err, exitcode)
    assert exitcode == 2
    assert b"odfdo-styles: error: the following arguments" in err
    assert b"usage" in err


# direct access to internal function


def test_styles_2_no_param_on_main_function(monkeypatch):
    with pytest.raises(SystemExit) as result:
        monkeypatch.setattr(sys, "argv", [])
        main_script()
        assert result.value.code >= 1


def test_styles_2_no_param():
    with pytest.raises(SystemExit) as result:
        params = parse_cli_args([])
        main_styles(params)
        assert result.value.code >= 1


def test_styles_2_version(capsys):
    with pytest.raises(SystemExit) as result:
        parse_cli_args(["--version"])
        assert result.value.code == 0
    captured = capsys.readouterr()

    assert "odfdo-styles v3" in captured.out


def test_styles_2_help(capsys):
    with pytest.raises(SystemExit) as result:
        parse_cli_args(["--help"])
        assert result.value.code == 0
    captured = capsys.readouterr()

    assert "Manipulate styles within" in captured.out


def test_styles_2_no_file():
    params = parse_cli_args(["none_file"])

    with pytest.raises(SystemExit) as result:
        main_styles(params)
        assert result.value.code >= 1


def test_styles_2_base_on_main_function(capsys, monkeypatch, samples):
    source = str(samples("base_text.odt"))
    monkeypatch.setattr(sys, "argv", ["odfdo-styles", source])

    main_script()
    captured = capsys.readouterr()

    assert "common used:y family:graphic" in captured.out
    assert "auto   used:y family:section" in captured.out


def test_styles_2_base(capsys, samples):
    source = str(samples("base_text.odt"))
    params = parse_cli_args([source])

    main_styles(params)
    captured = capsys.readouterr()

    assert "common used:y family:graphic" in captured.out
    assert "auto   used:y family:section" in captured.out


def test_styles_2_base_auto(capsys, samples):
    source = str(samples("base_text.odt"))
    params = parse_cli_args(["-a", source])

    main_styles(params)
    captured = capsys.readouterr()

    assert "auto   used:y family:drawing-page parent: name:Mdp1" in captured.out
    assert "common used" not in captured.out


def test_styles_2_base_common(capsys, samples):
    source = str(samples("base_text.odt"))
    params = parse_cli_args(["-c", source])

    main_styles(params)
    captured = capsys.readouterr()

    assert "auto   used" not in captured.out
    assert "common used" in captured.out


def test_styles_2_base_properties(capsys, samples):
    source = str(samples("base_text.odt"))
    params = parse_cli_args(["-p", source])

    main_styles(params)
    captured = capsys.readouterr()

    assert "- style:layout-grid-lines: 20" in captured.out
    assert "- style:text-underline-color: font-color" in captured.out


def test_styles_2_base_auto_properties(capsys, samples):
    source = str(samples("base_text.odt"))
    params = parse_cli_args(["-ap", source])

    main_styles(params)
    captured = capsys.readouterr()

    assert "- style:layout-grid-lines: 20" in captured.out
    assert "- style:text-underline-color: font-color" not in captured.out


def test_styles_2_base_common_properties(capsys, samples):
    source = str(samples("base_text.odt"))
    params = parse_cli_args(["-cp", source])

    main_styles(params)
    captured = capsys.readouterr()

    assert "- style:text-underline-color: font-color" in captured.out
    assert "- style:layout-grid-lines: 20" not in captured.out


def test_styles_2_delete_fail(capsys, samples):
    source = str(samples("base_text.odt"))
    params = parse_cli_args(["-d", source])

    with pytest.raises(SystemExit) as result:
        main_styles(params)
        assert result.value.code >= 1
    captured = capsys.readouterr()

    assert "Error: Will not delete in-place" in captured.err


def test_styles_2_delete_to_stdout(capsysbinary, samples):
    source = str(samples("base_text.odt"))
    params = parse_cli_args(["-d", "-o", "-", source])

    main_styles(params)
    captured = capsysbinary.readouterr()

    assert b"27 styles removed (0 error, 0 warning)" in captured.err
    assert len(captured.out) > 8000


def test_styles_2_delete_to_file(tmp_path, capsys, samples):
    source = str(samples("base_text.odt"))
    dest = tmp_path / "test_deleted.odt"
    params = parse_cli_args(["-d", "-o", f"{dest}", source])

    main_styles(params)
    captured = capsys.readouterr()

    assert "27 styles removed (0 error, 0 warning)" in captured.err
    assert dest.is_file()


def test_styles_2_show_to_file(tmp_path, capsys, samples):
    source = str(samples("base_text.odt"))
    dest = tmp_path / "styles.txt"
    params = parse_cli_args(["-o", f"{dest}", source])

    main_styles(params)
    captured = capsys.readouterr()

    assert not captured.out.strip()
    assert dest.is_file()


def test_styles_2_show_to_file2(tmp_path, samples):
    source = str(samples("base_text.odt"))
    dest = tmp_path / "styles2.txt"
    params = parse_cli_args(["-o", f"{dest}", source])

    main_styles(params)

    content = dest.read_text(encoding="utf-8")
    assert "common used:y family:graphic" in content
    assert "auto   used:y family:text" in content


def test_styles_2_show_odp(capsys, samples):
    source = str(samples("background.odp"))
    params = parse_cli_args([source])

    main_styles(params)
    captured = capsys.readouterr()

    assert "common used:y family:presentation" in captured.out
    assert "auto   used:n family:presentation" in captured.out


def test_styles_2_show_odp2(capsys, samples):
    source = str(samples("example.odp"))
    params = parse_cli_args([source])

    main_styles(params)
    captured = capsys.readouterr()

    assert "common used:y family:presentation" in captured.out
    assert "auto   used:n family:presentation" in captured.out


def test_styles_2_merge(tmp_path, capsys, samples):
    source = str(samples("base_text.odt"))
    source_styles = str(samples("lpod_styles.odt"))
    dest = tmp_path / "styles_text.odt"
    params = parse_cli_args(["-m", source_styles, "-o", f"{dest}", source])

    main_styles(params)
    captured = capsys.readouterr()

    assert "Done (0 error, 0 warning)" in captured.err
    assert not captured.out.strip()
    assert dest.is_file()


def test_styles_2_merge_prez(tmp_path, capsys, samples):
    source = str(samples("example.odp"))
    source_styles = str(samples("background.odp"))
    dest = tmp_path / "styles_text.odp"
    params = parse_cli_args(["-m", source_styles, "-o", f"{dest}", source])

    main_styles(params)
    captured = capsys.readouterr()

    assert "Done (0 error, 0 warning)" in captured.err



# Copyright 2018-2026 Jérôme Dumonteil
# Copyright (c) 2009-2010 Ars Aperta, Itaapy, Pierlis, Talend.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
#
# Authors (odfdo project): jerome.dumonteil@gmail.com
# The odfdo project is a derivative work of the lpod-python project:
# https://github.com/lpod/lpod-python
"""Collection of base templates used to create new ODF files."""



# Copyright 2018-2026 Jérôme Dumonteil
# Copyright (c) 2009-2010 Ars Aperta, Itaapy, Pierlis, Talend.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
#
# Authors (odfdo project): jerome.dumonteil@gmail.com
# The odfdo project is a derivative work of the lpod-python project:
# https://github.com/lpod/lpod-python
# Authors: Romain Gauthier <romain@itaapy.com>
#          Hervé Cauwelier <herve@itaapy.com>
#          David Versmisse <david.versmisse@itaapy.com>


from odfdo.element import Element
from odfdo.svg import SvgDescription, SvgTitle


def test_svg_title_class():
    title = SvgTitle()
    assert isinstance(title, SvgTitle)


def test_svg_title_minimal_tag():
    title = Element.from_tag("<svg:title/>")
    assert isinstance(title, SvgTitle)


def test_svg_title_name_init():
    title = SvgTitle("some title")
    assert title.title == "some title"


def test_svg_title_name_init_none():
    title = SvgTitle()
    assert title.title == ""


def test_svg_title_title_property():
    title = SvgTitle("first title")
    title.title = "second"
    assert title.title == "second"


def test_svg_desc_class():
    desc = SvgDescription()
    assert isinstance(desc, SvgDescription)


def test_svg_desc_minimal_tag():
    desc = Element.from_tag("<svg:desc/>")
    assert isinstance(desc, SvgDescription)


def test_svg_desc_name_init():
    desc = SvgDescription("some description")
    assert desc.description == "some description"


def test_svg_desc_name_init_none():
    desc = SvgDescription()
    assert desc.description == ""


def test_svg_desc_description_property():
    desc = SvgDescription("first description")
    desc.description = "second"
    assert desc.description == "second"



# Copyright 2018-2026 Jérôme Dumonteil
# Copyright (c) 2009-2010 Ars Aperta, Itaapy, Pierlis, Talend.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
#
# Authors (odfdo project): jerome.dumonteil@gmail.com
# The odfdo project is a derivative work of the lpod-python project:
# https://github.com/lpod/lpod-python
# Authors: Jerome Dumonteil <jerome.dumonteil@itaapy.com>

from collections.abc import Iterable

import pytest

from odfdo.document import Document
from odfdo.table import _XP_ROW_IDX, Table


@pytest.fixture
def table(samples) -> Iterable[Table]:
    document = Document(samples("big.ods"))
    yield document.body.tables[0]  # type: ignore


def test_count_height(table):
    assert table.height == 20100


def test_get_row_0(table):
    row = table.get_row(0)
    assert row.get_values() == [1]
    assert row.y == 0


def test_get_row_5099(table):
    row = table.get_row(5099)
    assert row.get_values() == [5100]
    assert row.y == 5099
    assert row.repeated in (None, 1)


def test_get_row_5100(table):
    row = table.get_row(5100)
    assert row.get_values() == ["same"]
    assert row.y == 5100
    assert row.repeated == 100


def test_get_row_5199(table):
    row = table.get_row(5199)
    assert row.get_values() == ["same"]
    assert row.y == 5199
    assert row.repeated == 100


def test_get_row_5200(table):
    row = table.get_row(5200)
    assert row.get_values() == [5101]
    assert row.y == 5200
    assert row.repeated in (None, 1)


def test_get_row_raw_5100(table):
    row = table._get_element_idx2(_XP_ROW_IDX, 5100)
    assert row.get_values() == ["same"]
    assert row.y is None
    assert row.repeated == 100


def test_get_row_raw_5101(table):
    row = table._get_element_idx2(_XP_ROW_IDX, 5101)
    print(row.serialize())
    assert row.get_values() == [5101]
    assert row.y is None
    assert row.repeated in (None, 1)


def test_get_row_raw_20099(table):
    row = table.get_row(20099)
    assert row.get_values() == [20000]
    assert row.y == 20099
    assert row.repeated in (None, 1)


def test_get_row_raw_20100(table):
    row = table.get_row(20100)
    assert row.get_values() == []
    assert row.y == 20100
    assert row.repeated in (None, 1)


def test_iter_rows_all(table):
    counter = 0
    for _row in table.iter_rows():
        counter += 1
    assert counter == 20100


def test_iter_rows_1(table):
    counter = 0
    for _row in table.iter_rows(0, 10000):
        counter += 1
    assert counter == 10001


def test_iter_rows_2(table):
    counter = 0
    for _row in table.iter_rows(1000, 50000):
        counter += 1
    assert counter == 19100


def test_iter_rows_3(table):
    counter = 0
    for _row in table.iter_rows(19000, 20000):
        counter += 1
    assert counter == 1001


def test_iter_rows_4(table):
    counter = 0
    for _row in table.iter_rows(19000, 19000):
        counter += 1
    assert counter == 1


def test_iter_rows_5(table):
    counter = 0
    for _row in table.iter_rows(10, 9):
        counter += 1
    assert counter == 0
